unit UOptions;

interface

uses
  Classes{TComponent},
  windows{TRect},
  Graphics{TFont},
  GmXml, dialogs;

type

  TWindowPosition = record
    WindowOuvert: boolean;
    WindowTop: integer;
    WindowLeft: integer;
    WindowWidth: integer;
    WindowHeight: integer;
    end;

  TOptions = class(TComponent)
  private
    {Internal variables, hiden  }
    FFichier, FFichierXml: string;            //Filename to store the object

    {Properties : Internal representation  }
    fCheminFichiers: string;
    fLangage: string;
    fCouleurFond: Integer;
    fPetitsBoutons: boolean;
    fAssociationFichiers: boolean;
    fWindow_Perspective: string;
    fWindow_VueDessus: string;
    fWindow_VueDroite: string;
    fWindow_VueFace: string;
    fWindow_VueGauche: string;
    fWindow_Anim: string;

    //fnReal: Double;
    //FFont: TFont;
    //faRect: Trect;
    //FUserName: string;
    //fLastUse: TdateTime;
    //fNoSplash: Boolean;
    //fLines: TStrings;//we always create this as TStringList

    {Procedures for validating data when assignig values to properties}
    procedure SetLangage(const Value: string);
    function GetCouleurFond: Integer;
    procedure SetWindow_Perspective(const Value: string);
    procedure SetWindow_VueDessus(const Value: string);
    procedure SetWindow_VueDroite(const Value: string);
    procedure SetWindow_VueFace(const Value: string);
    procedure SetWindow_VueGauche(const Value: string);
    procedure SetWindow_Anim(const Value: string);

    //procedure SetFont(const Value: TFont);
    //procedure SetUserName(const Value: string);
    //function GetScrennRes: TPoint;
    //procedure SetLines(const Value: TStrings);
  protected
    {This is ONLY to store properties which are not of type: integer, string, TStrings, TFont y similar}
    {If you work with string, TStrings, TFont and similar properties you dont need this procedures }
    //procedure DefineProperties(Filer: TFiler); override;
    //procedure ReadRect(Reader:Treader);
    //procedure WriteRect(Writer:Twriter);

  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;

    procedure ReadConfig;
    procedure SaveConfig;
    procedure ReadConfigXml;
    procedure SaveConfigXml;

    function tok(sep: string; var s: string): string;
    function WindowPositionToStr(WindowPosition: TWindowPosition): string;
    procedure StrToWindowPosition(str: string; var WindowPosition: TWindowPosition);
    //function FontToStr(Font: TFont): string;
    //procedure StrToFont(str: string; Font: TFont);

    {This property is not stored}
    //property ScrennRes: TPoint read GetScrennRes;
    property Fichier: string read FFichier write FFichier;
    property FichierXml: string read FFichierXml write FFichierXml;
  published
   {This properties are stored}
    property optCheminFichiers    : string   read fCheminFichiers write fCheminFichiers;
    property optLangage    : string   read fLangage write SetLangage;
    property optCouleurFond       : Integer   read GetCouleurFond write fCouleurFond;
    property optPetitsBoutons    : Boolean  read fPetitsBoutons write fPetitsBoutons;
    property optAssociationFichiers    : Boolean  read fAssociationFichiers write fAssociationFichiers;
    property optWindow_Perspective    : string   read fWindow_Perspective write SetWindow_Perspective;
    property optWindow_VueDessus    : string   read fWindow_VueDessus write SetWindow_VueDessus;
    property optWindow_VueDroite    : string   read fWindow_VueDroite write SetWindow_VueDroite;
    property optWindow_VueFace    : string   read fWindow_VueFace write SetWindow_VueFace;
    property optWindow_VueGauche    : string   read fWindow_VueGauche write SetWindow_VueGauche;
    property optWindow_Anim    : string   read fWindow_Anim write SetWindow_Anim;


    //property ARect       : TRect    read fARect    write fARect;
    //property NoSplash    : Boolean  read fNoSplash write fNoSplash;
    //property UserName    : string   read FUserName write SetUserName;
    //property nReal       : Double   read fnReal    write fnReal;
    //property Lines       : TStrings read fLines    write SetLines;
    //property Font        : TFont    read FFont     write SetFont;
    //property LastUse     : TDateTime read fLastUse write fLastUse;
  end;


var {Global, to use it from anywhere }
  Options: TOptions;
  XmlOptions: TGmXml;

implementation

uses
  SysUtils{ExtractFilePath},
  Forms{Screen};


{ TOptions }

{-----------------------------------------------------------------------------
  Algoritmo: TOptions.Create
  Comentarios: Create the internal objects, initialize properties to default values
-----------------------------------------------------------------------------}

constructor TOptions.Create(aOwner: TComponent);
begin
  inherited Create(AOWner);
  XmlOptions:= TGmXml.Create(AOWner);
  {Creating internal variables}
  //fLines    := TStringList.Create;// use always TStringList, never TStrings.
  //fFont     := Tfont.Create;

  //seting default values
  fLangage:= 'Franais.xml';
  fCouleurFond:= clBlue;
  fPetitsBoutons:= False;
  fAssociationFichiers:= True;
  fWindow_Perspective:= '144|6|220|220|y';
  fWindow_VueDessus:= '293|246|250|250|n';
  fWindow_VueDroite:= '145|528|250|250|n';
  fWindow_VueFace:= '188|251|250|250|y';
  fWindow_VueGauche:= '160|541|250|250|y';
  fWindow_Anim:= '184|38|258|368|n';


  //fLastUse := Now;

end;


destructor TOptions.Destroy;
begin
//Next line is optional, if omitted (deleted or commented) you must call SaveConfig
//to save the object
  //SaveConfig;

  {We must free what we create, before Object destruction}
  //fLines.Free;
  //fFont.Free;
  XmlOptions.Nodes.Clear;
  XmlOptions.Free;
  inherited Destroy;
end;

{-----------------------------------------------------------------------------
  Algoritmo: TOptions.SaveConfig
  Comentarios: Case UseRegistry of
    True: Store the object in windows registry and delete the file (Fichier)
    False:Store the object in the file (Fichier) and clear the registry.
-----------------------------------------------------------------------------}
procedure TOptions.SaveConfig;
begin
  //fLastUse := Now;
  WriteComponentResFile(Fichier, Self);
end;

procedure TOptions.SaveConfigXml;
begin
  XmlOptions.Nodes.Clear;
  with XmlOptions.Nodes do begin
    AddOpenTag('Config');

    AddOpenTag('optCheminFichiers').AsString := optCheminFichiers;
    AddCloseTag;
    AddOpenTag('optLangage').AsString := optLangage;
    AddCloseTag;
    AddOpenTag('optCouleurFond').AsInteger := optCouleurFond;
    AddCloseTag;
    AddOpenTag('optPetitsBoutons').AsBoolean := optPetitsBoutons;
    AddCloseTag;
    AddOpenTag('optAssociationFichiers').AsBoolean := optAssociationFichiers;
    AddCloseTag;

    AddOpenTag('Windows');
    AddOpenTag('optWindow_Perspective').AsString := optWindow_Perspective;
    AddCloseTag;
    AddOpenTag('optWindow_VueDessus').AsString := optWindow_VueDessus;
    AddCloseTag;
    AddOpenTag('optWindow_VueDroite').AsString := optWindow_VueDroite;
    AddCloseTag;
    AddOpenTag('optWindow_VueFace').AsString := optWindow_VueFace;
    AddCloseTag;
    AddOpenTag('optWindow_VueGauche').AsString := optWindow_VueGauche;
    AddCloseTag;
    AddOpenTag('optWindow_Anim').AsString := optWindow_Anim;
    AddCloseTag;
    AddCloseTag;

    AddCloseTag;
    end;
  XmlOptions.SaveToFile(FichierXml);
end;

procedure TOptions.SetLangage(const Value: string);
begin
  if Value <> ''
    then fLangage:= Value
    else fLangage:= 'Franais.xml';
end;

function TOptions.GetCouleurFond: Integer;
begin
  Result:= ABS(FCouleurFond);
end;



function TOptions.tok(sep: string; var s: string): string;
  function IsOneOf(c, s: string): Boolean;
  var
    iTmp: integer;
  begin
    Result:= False;
    for iTmp:= 1 to Length(s) do begin
      if c = Copy(s, iTmp, 1) then begin
        Result:= True;
        Exit;
        end;
      end;
  end;
var
  c, t: string;
begin
  if s = '' then begin
    Result:= s;
    Exit;
    end;
  c:= Copy(s, 1, 1);
  while IsOneOf(c, sep) do begin
    s:= Copy(s, 2, Length(s) - 1);
    c:= Copy(s, 1, 1);
    end;
  t:= '';
  while (not IsOneOf(c, sep)) and (s <> '') do begin
    t:= t + c;
    s:= Copy(s, 2, length(s)-1);
    c:= Copy(s, 1, 1);
    end;
  Result:= t;
end;

function TOptions.WindowPositionToStr(WindowPosition: TWindowPosition): string;
begin
  Result:= '';
  Result:= Result + IntToStr(WindowPosition.WindowTop) + '|';
  Result:= Result + IntToStr(WindowPosition.WindowLeft) + '|';
  Result:= Result + IntToStr(WindowPosition.WindowWidth) + '|';
  Result:= Result + IntToStr(WindowPosition.WindowHeight) + '|';
  if WindowPosition.WindowOuvert
    then Result:= Result + 'y'
    else Result:= Result + 'n';
end;

procedure TOptions.StrToWindowPosition(str: string; var WindowPosition: TWindowPosition);
var
  a: string;
begin
  if str = '' then exit;
  a:= str;
  WindowPosition.WindowTop:= StrToInt(tok('|', a));
  WindowPosition.WindowLeft:= StrToInt(tok('|', a));
  WindowPosition.WindowWidth:= StrToInt(tok('|', a));
  WindowPosition.WindowHeight:= StrToInt(tok('|', a));
  WindowPosition.WindowOuvert:= (a[2] = 'y');
end;

procedure TOptions.SetWindow_Perspective(const Value: string);
begin
  if Value <> ''
    then fWindow_Perspective:= Value
    else fWindow_Perspective:= '144|6|220|220|y';
end;

procedure TOptions.SetWindow_VueDessus(const Value: string);
begin
  if Value <> ''
    then fWindow_VueDessus:= Value
    else fWindow_VueDessus:= '293|246|250|250|n';
end;

procedure TOptions.SetWindow_VueDroite(const Value: string);
begin
  if Value <> ''
    then fWindow_VueDroite:= Value
    else fWindow_VueDroite:= '145|528|250|250|n';
end;

procedure TOptions.SetWindow_VueFace(const Value: string);
begin
  if Value <> ''
    then fWindow_VueFace:= Value
    else fWindow_VueFace:= '188|251|220|220|y';
end;

procedure TOptions.SetWindow_VueGauche(const Value: string);
begin
  if Value <> ''
    then fWindow_VueGauche:= Value
    else fWindow_VueGauche:= '160|541|250|250|y';
end;

procedure TOptions.SetWindow_Anim(const Value: string);
begin
  if Value <> ''
    then fWindow_Anim:= Value
    else fWindow_Anim:= '184|38|258|368|n';
end;







{
function TOptions.FontToStr(Font: TFont): string;
  procedure yes(var str: string);
  begin
    str:= str + 'y';
  end;
  procedure no(var str: string);
  begin
    str:= str + 'n';
  end;
begin
  Result:= '';
  Result:= Result + IntToStr(Font.Color) + '|';
  Result:= Result + IntToStr(Font.Height) + '|';
  Result:= Result + Font.Name + '|';
  Result:= Result + IntToStr(Ord(Font.Pitch)) + '|';
  Result:= Result + IntToStr(Font.PixelsPerInch) + '|';
  Result:= Result + IntToStr(Font.size) + '|';
  if fsBold in Font.style then yes(Result) else no(Result);
  if fsItalic in Font.style then yes(Result) else no(Result);
  if fsUnderline in Font.style then yes(Result) else no(Result);
  if fsStrikeout in Font.style then yes(Result) else no(Result);
end;

procedure TOptions.StrToFont(str: string; Font: TFont);
begin
  if str = '' then exit;
  Font.Color:= StrToInt(tok('|', str));
  Font.Height:= StrToInt(tok('|', str));
  Font.Name:= tok('|', str);
  Font.Pitch:= TFontPitch(StrToInt(tok('|', str)));
  Font.PixelsPerInch:= StrToInt(tok('|', str));
  Font.Size:= StrToInt(tok('|', str));
  Font.Style:= [];
  if str[2] = 'y' then Font.Style:= Font.Style + [fsBold];
  if str[3] = 'y' then Font.Style:= Font.Style + [fsItalic];
  if str[4] = 'y' then Font.Style:= Font.Style + [fsUnderline];
  if str[5] = 'y' then Font.Style:= Font.Style + [fsStrikeout];
end;
}


























{-----------------------------------------------------------------------------
  Algoritmo: TOptions.GetScrennRes
  Comentarios: Returns the actual ScrennResolution. X = width; Y = height
-----------------------------------------------------------------------------}
{
function TOptions.GetScrennRes: Tpoint;
begin
   with Result do
      begin
      x := Screen.width;
      y := Screen.Height;
      end;
end;

procedure TOptions.DefineProperties(Filer: TFiler);
begin
  inherited;
{The property is stored with this name: 'ARECT'
 Which procedure read it : ReadRect
 Which procedure write it: WriteRect
 for more help see online help }
{
  Filer.DefineProperty('ARECT', ReadRect, WriteRect, True);
end;

{-----------------------------------------------------------------------------
  Algoritmo: TOptions.ReadRect
  Comentarios: Internal use. Read the property from where is stored
		(.dfm, external file, stream, etc)
-----------------------------------------------------------------------------}
{
procedure TOptions.ReadRect(Reader: Treader);
begin
//Remember: use Farect (not Arect)
  with FARect, Reader do
    begin
    ReadListBegin;
    Left   := ReadInteger;
    Top    := ReadInteger;
    Right  := ReadInteger;
    Bottom := ReadInteger;
    ReadListEnd;
    end;
end;

{-----------------------------------------------------------------------------
  Algoritmo: TOptions.WriteRect
  Comentarios: Complemento de ReadRect.
-----------------------------------------------------------------------------}
{
procedure TOptions.WriteRect(Writer: Twriter);
begin
//Remember: use Farect (not Arect)
  with fARect, Writer do
   begin
   WriteListBegin;
   WriteInteger(Left);
   WriteInteger(top);
   WriteInteger(Right);
   WriteInteger(Bottom);
   WriteListEnd;
   end;
end;




{-----------------------------------------------------------------------------
  Algoritmo: TOptions.SetFont
  Comentarios: We must use FFont and never Font.
               When its possible we must use Assign()
-----------------------------------------------------------------------------}
{
procedure TOptions.SetFont(const Value: TFont);
begin
  FFont.assign(Value);
end;

{-----------------------------------------------------------------------------
  Algoritmo: TOptions.SetUserName
  Comentarios: Value is the new UserName
   The instruction :
                              Options.UserName := 'Papillon';
   The compiler translate it in :
                              Options.SetUserName('Papillon');

-----------------------------------------------------------------------------}
{
procedure TOptions.SetUserName(const Value: string);
begin
  if Value <> '' then
    FUserName := Value
  else
//in place of :
    fUserName := '--Not set--';
// we could use :
//    raise Exception.Create('UserName must have a value');

end;

{-----------------------------------------------------------------------------
  Algoritmo: TOptions.ReadConfig
  Comentarios: Case UseRegistry of
	True: try to Read from registry
        False: if the file exists try to read from the file
-----------------------------------------------------------------------------}
procedure TOptions.ReadConfig;
var S: string;
begin
  if fileexists(Fichier) then
    try
      Self:= TOptions(ReadComponentResFile(Fichier, Self));
    except
      //If you like, you can tell the user "I cant read options"
      end;
ReadConfigXml;
end;

procedure TOptions.ReadConfigXml;
var
  GmXmlNode: TGmXmlNode;
  a: string;
begin
  if fileexists(FichierXml) then
    try
      XmlOptions.LoadFromFile(FichierXml);

      GmXmlNode:= XmlOptions.GetNodeByPath('optCheminFichiers');
      if GmXmlNode <> nil then optCheminFichiers:= GmXmlNode.AsString;
      GmXmlNode:= XmlOptions.GetNodeByPath('optLangage');
      if GmXmlNode <> nil then optLangage:= GmXmlNode.AsString;
      GmXmlNode:= XmlOptions.GetNodeByPath('optCouleurFond');
      if GmXmlNode <> nil then optCouleurFond:= GmXmlNode.AsInteger;
      GmXmlNode:= XmlOptions.GetNodeByPath('optPetitsBoutons');
      if GmXmlNode <> nil then optPetitsBoutons:= GmXmlNode.AsBoolean;
      GmXmlNode:= XmlOptions.GetNodeByPath('optAssociationFichiers');
      if GmXmlNode <> nil then optAssociationFichiers:= GmXmlNode.AsBoolean;

      GmXmlNode:= XmlOptions.GetNodeByPath('Windows\optWindow_Perspective');
      if GmXmlNode <> nil then optWindow_Perspective:= GmXmlNode.AsString;
      GmXmlNode:= XmlOptions.GetNodeByPath('Windows\optWindow_VueDessus');
      if GmXmlNode <> nil then optWindow_VueDessus:= GmXmlNode.AsString;
      GmXmlNode:= XmlOptions.GetNodeByPath('Windows\optWindow_VueDroite');
      if GmXmlNode <> nil then optWindow_VueDroite:= GmXmlNode.AsString;
      GmXmlNode:= XmlOptions.GetNodeByPath('Windows\optWindow_VueFace');
      if GmXmlNode <> nil then optWindow_VueFace:= GmXmlNode.AsString;
      GmXmlNode:= XmlOptions.GetNodeByPath('Windows\optWindow_VueGauche');
      if GmXmlNode <> nil then optWindow_VueGauche:= GmXmlNode.AsString;
      GmXmlNode:= XmlOptions.GetNodeByPath('Windows\optWindow_Anim');
      if GmXmlNode <> nil then optWindow_Anim:= GmXmlNode.AsString;
    except
      //If you like, you can tell the user "I cant read options"
      showmessage('aa');
      end;

end;

{-----------------------------------------------------------------------------
  Algoritmo: TOptions.SetLines
  Comentarios: Im using fLines in place of Lines to avoid recursive calls
               We use Assign() to copy Value to fLines
-----------------------------------------------------------------------------}
{procedure TOptions.SetLines(const Value: TStrings);
begin
  fLines.Assign(Value);
end;
}

end.

