(******************************************************************************
Copyright (C) 2003 Pascal Peter

http://pascal.peter.free.fr/
pascal.peter@free.fr

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
modifier conformment aux dispositions de la Licence Publique Gnrale GNU,
telle que publie par la Free Software Foundation ; version 2 de la licence,
ou encore toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
voir la Licence Publique Gnrale GNU.

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme
temps que ce programme ; si ce n'est pas le cas,
crivez  la Free Software Foundation Inc.,
675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
-------------------------------------------------------------------------------

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************)

unit Animations;


{
Utilisation :

1.dclarer une variable
  ObjActor: TObjActor;
2.crer le TObjActor (dans FormCreate)
  ObjActor:= TObjActor(TGLBaseSceneObject(World).AddNewChild(TObjActor));
  ObjActor.LoadFromFile(CheminFichiers + '000.anm');
3.le faire vivre (dans GLCadencer1Progress)
  if ObjActor <> nil then ObjActor.Progress(deltaTime, newTime);
4.faire ce qui faut quand y faut
  ObjActor.CoeffScale:= 0.7;
  ObjActor.ObjAnimations.JoueAnimationViaNom('saute');
  ObjActor.ObjAnimations.Vitesse:= 3;
  ObjActor.ObjAnimations.EnAttente:= True;
  etc...
5.le librer en quittant (dans FormCloseQuery)
  ObjActor.free;
}




interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms,

  GLScene, GLObjects, VectorGeometry, GLVectorFileObjects,
  GeometryBB,


  MMSystem,       dialogs,

  glObjet3ds,

  GmXml, object2xml;


const
  MaxLengthString = 20;
  NouvelleAnimation = 'Nouvelle Animation';

  // to be used as descriptive indices
  X = 0;
  Y = 1;
  Z = 2;

  EnBas = -1;
  AuCentre = 0;
  EnHaut = 1;

  EPSILON3 = 0.001;

type


  TNameString = string[MaxLengthString];
  TNomFichier = string[250];

  TObjActuel = record
    IndexAnimation, IndexFrame: integer;
    NomAnimation: string;
    OK: boolean;
    TouteLAnimation: boolean;
    end;

  TObjValeurs = record
    Position:     TVector;
    Direction:    TVector;
    Up:           TVector;
    Scale:        TVector;
    Couleur:      TVector;
    Transparence: double;
    //TextureDisabled: boolean;
    end;

  TObjSimple = class(TObject)
    private
      FObjValeurs       : TObjValeurs;
      FFils             : TList;
    public
      GLObj: TGLBaseSceneObject;
      constructor Create;
      procedure Initialise;
      destructor Destroy; override;
      procedure FromGLObj;
      procedure ToGLObj;
    published
      property ObjValeurs:      TObjValeurs   read FObjValeurs      write FObjValeurs;
      property Fils:            TList         read FFils            write FFils;
    end;

  TObjFrame = class(TObject)
    private
      FComponents: TList;
      FDuree:      integer;
    public
      constructor Create;
      destructor Destroy; override;
      procedure Initialiser;

      procedure AjouteComponent(Component: TComponent);
      procedure SupprimeComponent(Component: TGLBaseSceneObject);
    published
      property Components: TList   read FComponents write FComponents;
      property Duree:      integer read FDuree      write FDuree;
    end;

  TObjAnimation = class(TObject)
    private
      FNom              : TNameString;
      FLoop             : boolean;
      FSuivante         : integer;
      FNumFrameSon      : integer;
      FVitesse          : single;
      FCurrentFrame     : integer;
    public
      Frames:                 TList;
      MemoryStreamFichierSon: TMemoryStream;

      constructor Create;
      destructor Destroy; override;

      procedure Initialiser;
      procedure PlaceFichierSon(FichierSon: string; NumFrame: integer);
      procedure SupprimeSon;

      procedure SupprimeComponent(Component: TGLBaseSceneObject);

      procedure SupprimerFrame(IndexFrameASupprimer: integer);
    published
      property Nom:          TNameString read FNom          write FNom;
      property Loop:         boolean     read FLoop         write FLoop;
      property Suivante:     integer     read FSuivante     write FSuivante;
      property NumFrameSon:  integer     read FNumFrameSon  write FNumFrameSon;
      property Vitesse:      single      read FVitesse      write FVitesse;
      property CurrentFrame: integer     read FCurrentFrame write FCurrentFrame;
    end;

  TObjAnimations = class(TObject)
    private
      FCurrentAnimation  : integer;
      FVitesse           : single;
      FExTime            : Double;
      FExT               : single;
      FGLParent          : TGLBaseSceneObject;
      FNomsDesAnimations : TStringList;
    protected
      procedure SetCurrentAnimation(Value: integer);
      procedure SetVitesse(Value: single);
    public
      Animations               : TList;
      EnAttente                : boolean;
      IsSimple                 : boolean;
      ObjFrameDeBase           : TObjFrame;
      constructor Create;
      destructor Destroy; override;
      procedure Initialiser;
      procedure Progress(const deltaTime, newTime : Double);
      procedure ChangeVisible(Animation1, Frame1,
                              Animation2, Frame2: integer;
                              t: single);
      procedure MetAJourNomsDesAnimations;
      function JoueAnimationViaNom(NomAnimation: string): boolean;
      function JoueAnimationViaNomVitesse(NomAnimation: string; Coeff: single): boolean;
      function RetourneNomAnimationActuelle: string;
      procedure Affiche(Animation, Frame: integer);
      function RetourneActuel: TObjActuel;
      function AnimationActuelle: TObjAnimation;
      procedure AjouteAnimation(var NewNom: string);
      procedure SupprimerAnimation(IndexAnimationASupprimer: integer);
      procedure Deplace(IndexAnimationADeplacer, IndexFrameADeplacer,
                        IndexAnimationCible, IndexFrameCible: integer);
      procedure FromGLObj;
      procedure ToGLObj;
      procedure MetEnPlaceLesGLObj;
      procedure Simplifier(PourEnregistrer: boolean);
      procedure Compliquer;
      procedure SupprimeComponent(Component: TGLBaseSceneObject);
      procedure SaveToStream(Stream: TStream);
      procedure SaveToXML(XML: TGmXML);
      procedure LoadFromStream(Stream: TStream);
      procedure LoadFromXML(XML: TGmXML);

      procedure gloups;
    published
      property CurrentAnimation:  integer            read FCurrentAnimation   write SetCurrentAnimation;
      property Vitesse:           single             read FVitesse            write SetVitesse;
      property ExTime:            Double             read FExTime             write FExTime;
      property GLParent:          TGLBaseSceneObject read FGLParent           write FGLParent;
      property NomsDesAnimations: TStringList        read FNomsDesAnimations  write FNomsDesAnimations;
    end;

  TObjActor = class(TGLDummyCube)
    private
      FObjAnimations: TObjAnimations;
      FCoeffScale       : single;
    protected
      procedure SetCoeffScale(Value: single);
    public
      AlignementVertical       : integer;
      minmax                   : TVector;
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      procedure Initialiser;
      procedure Progress(const deltaTime, newTime : Double);
      procedure CalculeMinMax;
      procedure LoadFromFile(const FileName: string; AvecPBar: boolean);
    published
      property ObjAnimations: TObjAnimations read FObjAnimations write FObjAnimations;
      property CoeffScale:    single         read FCoeffScale    write SetCoeffScale;
    end;

  procedure RecupereValeurs(ooo: TGLBaseSceneObject; var ObjSimple: TObjSimple);
  procedure AppliqueValeurs(var ooo: TGLBaseSceneObject; ObjSimple: TObjSimple);
  procedure ValeursLerp(ObjSimple1, ObjSimple2: TObjSimple; var ObjSimple: TObjSimple; t: Single);
  procedure RecopieObjSimple(ObjSimple1: TObjSimple; var ObjSimple: TObjSimple);
  function TrouveObjSimpleParGLObjName(AName: string; List: TList;
                                       ExplorerSousListes: boolean;
                                       IndexSiNameVide: integer): TObjSimple;

implementation

uses Utils;



procedure RecupereValeurs(ooo: TGLBaseSceneObject; var ObjSimple: TObjSimple);

    procedure Recursion(var ObjSimple: TObjSimple; GLBaseSceneObject: TGLBaseSceneObject);
    var
      i: integer;
      Newooo: TGLBaseSceneObject;
      NewObjSimple: TObjSimple;
    begin
      with ObjSimple.ObjValeurs do begin
        Position:=     GLBaseSceneObject.Position.AsVector;
        Direction:=    GLBaseSceneObject.Direction.AsVector;
        Up:=           GLBaseSceneObject.Up.AsVector;
        Scale:=        GLBaseSceneObject.Scale.AsVector;
        end;
      ObjSimple.GLObj:= GLBaseSceneObject;
      if GLBaseSceneObject is TGLCustomSceneObject
        then with ObjSimple.ObjValeurs do begin
          Couleur:=      TGLCustomSceneObject(GLBaseSceneObject).Material.FrontProperties.Diffuse.Color;
          Transparence:= TGLCustomSceneObject(GLBaseSceneObject).Material.FrontProperties.Diffuse.Alpha;
          //TextureDisabled:= TGLCustomSceneObject(GLBaseSceneObject).Material.Texture.disabled;
          end;
      if GLBaseSceneObject is TObjet3ds then exit;
      if GLBaseSceneObject is TGLBaseMesh then exit;
      if GLBaseSceneObject.Count > 0 then
        for i:= 0 to GLBaseSceneObject.Count - 1 do begin
          Newooo:= GLBaseSceneObject.Children[i];
          NewObjSimple:= TObjSimple.Create;
          Recursion(NewObjSimple, Newooo);
          ObjSimple.Fils.Add(NewObjSimple);
          end;
    end;

begin
  ObjSimple.Initialise;
  Recursion(ObjSimple, ooo);
end;

procedure AppliqueValeurs(var ooo: TGLBaseSceneObject; ObjSimple: TObjSimple);
    procedure Recursion(var GLBaseSceneObject: TGLBaseSceneObject; ObjSimple: TObjSimple);
    var
      i: integer;
      Newooo: TGLBaseSceneObject;
      NewObjSimple: TObjSimple;
    begin
      with GLBaseSceneObject do begin
        Position.AsVector:=     ObjSimple.ObjValeurs.Position;
        Direction.AsVector:=    ObjSimple.ObjValeurs.Direction;
        Up.AsVector:=           ObjSimple.ObjValeurs.Up;
        Scale.AsVector:=        ObjSimple.ObjValeurs.Scale;
        end;
      if GLBaseSceneObject is TGLCustomSceneObject then
        with TGLCustomSceneObject(GLBaseSceneObject) do begin
          Material.FrontProperties.Diffuse.Color:=      ObjSimple.ObjValeurs.Couleur;
          Material.FrontProperties.Diffuse.Alpha:= ObjSimple.ObjValeurs.Transparence;
          {Material.Texture.disabled:= ObjSimple.ObjValeurs.TextureEnabled;
          if ObjSimple.ObjValeurs.TextureDisabled
            then TGLPersistentImage(Material.Texture.Image).Picture.Graphic:= nil;}
          end;
      if ObjSimple.Fils.Count > 0 then
        for i:= 0 to ObjSimple.Fils.Count - 1 do begin
          NewObjSimple:= ObjSimple.Fils[i];
          Newooo:= NewObjSimple.GLObj;
          Recursion(Newooo, NewObjSimple);
          end;
    end;
begin
  Recursion(ooo, ObjSimple);
end;

procedure ValeursLerp(ObjSimple1, ObjSimple2: TObjSimple; var ObjSimple: TObjSimple; t: Single);
    procedure Recursion(ObjSimple1, ObjSimple2: TObjSimple; ObjSimple: TObjSimple);
    var
      i, j: integer;
      a: string;
      NewObjSimple1: TObjSimple;
    begin
      with ObjSimple.ObjValeurs do begin
        Position:= VectorLerp(ObjSimple1.ObjValeurs.Position, ObjSimple2.ObjValeurs.Position, t);
        Direction:= VectorLerp(ObjSimple1.ObjValeurs.Direction, ObjSimple2.ObjValeurs.Direction, t);
        Up:= VectorLerp(ObjSimple1.ObjValeurs.Up, ObjSimple2.ObjValeurs.Up, t);
        Scale:= VectorLerp(ObjSimple1.ObjValeurs.Scale, ObjSimple2.ObjValeurs.Scale, t);
        Couleur:= VectorLerp(ObjSimple1.ObjValeurs.Couleur, ObjSimple2.ObjValeurs.Couleur, t);
        Transparence:= ObjSimple1.ObjValeurs.Transparence + (ObjSimple2.ObjValeurs.Transparence - ObjSimple1.ObjValeurs.Transparence) * t;
        end;
      if ObjSimple2.Fils.Count > 0 then
        for i:= 0 to ObjSimple2.Fils.Count - 1 do begin
          a:= TObjSimple(ObjSimple2.Fils[i]).GLObj.Name;
          NewObjSimple1:= TrouveObjSimpleParGLObjName(a, ObjSimple1.Fils, False, i);
          j:= ObjSimple1.Fils.IndexOf(NewObjSimple1);
          Recursion(NewObjSimple1,
                    ObjSimple2.Fils[i],
                    ObjSimple.Fils[j]);
          end;
    end;
begin
  RecopieObjSimple(ObjSimple1, ObjSimple);
  Recursion(ObjSimple1, ObjSimple2, ObjSimple);
end;

procedure RecopieObjSimple(ObjSimple1: TObjSimple; var ObjSimple: TObjSimple);
var
  i: integer;
  NewObjSimple: TObjSimple;
begin
  ObjSimple.Initialise;
  ObjSimple.ObjValeurs:= ObjSimple1.ObjValeurs;
  ObjSimple.GLObj:= ObjSimple1.GLObj;
  if ObjSimple1.Fils.Count > 0 then
    for i:= 0 to ObjSimple1.Fils.Count - 1 do begin
      NewObjSimple:= TObjSimple.Create;
      ObjSimple.Fils.Add(NewObjSimple);
      RecopieObjSimple(ObjSimple1.Fils[i], NewObjSimple);
      end;
end;

function TrouveObjSimpleParGLObjName(AName: string; List: TList;
                                     ExplorerSousListes: boolean;
                                     IndexSiNameVide: integer): TObjSimple;
var
  i, j: integer;
  ObjSimple: TObjSimple;
begin
  Result:= nil;
  if AName = '' then
    if TObject(List[IndexSiNameVide]) is TObjSimple then begin
      Result:= TObjSimple(List[IndexSiNameVide]);
      exit;
      end;
  if List.Count > 0 then
    for i:= 0 to List.Count - 1 do
      if TObject(List[i]) is TObjSimple then begin
        ObjSimple:= TObjSimple(List[i]);
        if ObjSimple.GLObj.Name = AName then begin
          Result:= ObjSimple;
          exit;
          end;
        if ExplorerSousListes then
          if ObjSimple.Fils.Count > 0 then
            for j:= 0 to ObjSimple.Fils.Count - 1 do begin
              Result:= TrouveObjSimpleParGLObjName(AName, ObjSimple.Fils, ExplorerSousListes, j);
              if Result <> nil then exit;
              end;
        end;
end;

















// ------------------
// ------------------ TObjSimple --------------
// ------------------
constructor TObjSimple.Create;
begin
  inherited Create;
  FFils:= TList.create;
  GLObj:= nil;
end;

procedure TObjSimple.Initialise;
var
  i: integer;
begin
  if FFils.Count > 0 then
    for i:= FFils.Count - 1 downto 0 do
      TObjSimple(FFils[i]).free;
  FFils.Clear;
end;

destructor TObjSimple.Destroy;
begin
  Initialise;
  FFils.free;
  GLObj:= nil;
  inherited Destroy;
end;

procedure TObjSimple.FromGLObj;
var
  i: integer;
  NewObjSimple: TObjSimple;
begin
  Initialise;

  with ObjValeurs do begin
    Position:=     GLObj.Position.AsVector;
    Direction:=    GLObj.Direction.AsVector;
    Up:=           GLObj.Up.AsVector;
    Scale:=        GLObj.Scale.AsVector;
    if GLObj is TGLCustomSceneObject then begin
      Couleur:=      TGLCustomSceneObject(GLObj).Material.FrontProperties.Diffuse.Color;
      Transparence:= TGLCustomSceneObject(GLObj).Material.FrontProperties.Diffuse.Alpha;
      //TextureDisabled:= TGLCustomSceneObject(GLObj).Material.Texture.disabled;
      end;
    end;
  if GLObj is TObjet3ds then exit;
  if GLObj is TGLBaseMesh then exit;
  if GLObj.Count > 0 then
    for i:= 0 to GLObj.Count - 1 do begin
      NewObjSimple:= TObjSimple.Create;
      NewObjSimple.GLObj:= GLObj.Children[i];
      Fils.Add(NewObjSimple);
      NewObjSimple.FromGLObj;
      end;

end;

procedure TObjSimple.ToGLObj;
var
  i: integer;
begin
  GLObj.Position.AsVector:= ObjValeurs.Position;
  GLObj.Direction.AsVector:= ObjValeurs.Direction;
  GLObj.Up.AsVector:= ObjValeurs.Up;
  GLObj.Scale.AsVector:= ObjValeurs.Scale;
  if GLObj is TGLCustomSceneObject then begin
    TGLCustomSceneObject(GLObj).Material.FrontProperties.Diffuse.Color:= ObjValeurs.Couleur;
    TGLCustomSceneObject(GLObj).Material.FrontProperties.Diffuse.Alpha:= ObjValeurs.Transparence;
    //TGLCustomSceneObject(GLObj).Material.Texture.disabled:= ObjValeurs.TextureDisabled;
    end;
  if GLObj is TObjet3ds then exit;
  if GLObj is TGLBaseMesh then exit;
  if Fils.Count > 0 then
    for i:= 0 to Fils.Count - 1 do
      TObjSimple(Fils[i]).ToGLObj;
end;



// ------------------
// ------------------ TObjFrame --------------
// ------------------
constructor TObjFrame.Create;
begin
  inherited Create;
  FComponents:= TList.create;
  FDuree:= 1000;
end;

procedure TObjFrame.Initialiser;
var
  i: integer;
begin
  if FComponents.Count > 0 then
    for i:= FComponents.Count - 1 downto 0 do
      TObjSimple(FComponents[i]).free;
  FComponents.Clear;
end;


destructor TObjFrame.Destroy;
begin
  Initialiser;
  FComponents.free;
  inherited Destroy;
end;

procedure TObjFrame.AjouteComponent(Component: TComponent);
var
  NewObjSimple: TObjSimple;
begin
  NewObjSimple:= TObjSimple.create;
  NewObjSimple.GLObj:= TGLBaseSceneObject(Component);
  FComponents.Add(NewObjSimple);
  NewObjSimple.FromGLObj;
end;

procedure TObjFrame.SupprimeComponent(Component: TGLBaseSceneObject);

    procedure SupprimeObjSimpleDesListes(ObjSimple: TObjSimple; List: TList);
    var
      i: integer;
    begin
      if List.IndexOf(ObjSimple) > -1 then begin
        List.Delete(List.IndexOf(ObjSimple));
        exit;
        end;
      if List.Count > 0 then
        for i:= 0 to List.Count - 1 do
          if TObject(List[i]) is TObjSimple then
            SupprimeObjSimpleDesListes(ObjSimple, TObjSimple(List[i]).Fils);
    end;

var
  i, j: integer;
  ObjSimple: TObjSimple;
begin
  ObjSimple:= TrouveObjSimpleParGLObjName(Component.Name, Components, True, 0);
  if ObjSimple <> nil then begin
    SupprimeObjSimpleDesListes(ObjSimple, Components);
    ObjSimple.Free;
    end;
end;

// ------------------
// ------------------ TObjAnimation --------------
// ------------------
constructor TObjAnimation.Create;
begin
  inherited Create;
  Frames:= TList.create;
  MemoryStreamFichierSon:= TMemoryStream.create;
  FNom:= NouvelleAnimation;
  FLoop:= False;
  FSuivante:= 0;
  FNumFrameSon:= -1;
  FVitesse:= 1;
  FCurrentFrame:= 0;
end;

destructor TObjAnimation.Destroy;
begin
  Initialiser;
  Frames.free;
  MemoryStreamFichierSon.free;
  inherited Destroy;
end;

procedure TObjAnimation.Initialiser;
var
  i: integer;
begin
  if Frames.Count > 0 then
    for i:= Frames.Count - 1 downto 0 do
      TObjFrame(Frames[i]).free;
  Frames.Clear;
  FNumFrameSon:= -1;
  FCurrentFrame:= 0;
  MemoryStreamFichierSon.clear;
end;

procedure TObjAnimation.PlaceFichierSon(FichierSon: string;
                                        NumFrame: integer);
var
  FileStream: TFileStream;
begin
  FileStream:= TFileStream.Create(FichierSon, fmOpenRead);
  try
    MemoryStreamFichierSon.clear;
    MemoryStreamFichierSon.free;
    MemoryStreamFichierSon:= TMemoryStream.create;
    MemoryStreamFichierSon.CopyFrom(FileStream, FileStream.Size);
  finally
    FileStream.free;
    end;
  FNumFrameSon:= NumFrame;
end;

procedure TObjAnimation.SupprimeSon;
begin
  MemoryStreamFichierSon.clear;
  FNumFrameSon:= -1;
end;

procedure TObjAnimation.SupprimeComponent(Component: TGLBaseSceneObject);
var
  i: integer;
begin
  if Frames.Count > 0 then
    for i:= Frames.Count - 1 downto 0 do
      TObjFrame(Frames[i]).SupprimeComponent(Component);
end;


procedure TObjAnimation.SupprimerFrame(IndexFrameASupprimer: integer);
begin
  if Frames.Count > 0 then begin
    if FNumFrameSon = IndexFrameASupprimer then begin
      MemoryStreamFichierSon.clear;
      FNumFrameSon:= - 1;
      end;
    if FNumFrameSon > IndexFrameASupprimer then FNumFrameSon:= FNumFrameSon - 1;
    if FCurrentFrame >= IndexFrameASupprimer then FCurrentFrame:= FCurrentFrame - 1;
    if FCurrentFrame > 0 then FCurrentFrame:= 0;
    TObjFrame(Frames[IndexFrameASupprimer]).free;
    Frames.Delete(IndexFrameASupprimer);
    end;
end;

// ------------------
// ------------------ TObjAnimations ------------------
// ------------------
constructor TObjAnimations.Create;
begin
  inherited Create;
  Animations:= TList.create;
  FNomsDesAnimations:= TStringList.Create;
  ObjFrameDeBase:= TObjFrame.create;
  Initialiser;
end;

destructor TObjAnimations.Destroy;
begin
  Initialiser;
  EnAttente:= True;
  Animations.free;
  FNomsDesAnimations.Free;
  ObjFrameDeBase.Free;
  inherited Destroy;
end;

procedure TObjAnimations.Initialiser;
var
  i: integer;
begin
  EnAttente:= True;
  if Animations.Count > 0 then
    for i:= Animations.Count - 1 downto 0 do
      TObjAnimation(Animations[i]).free;
  Animations.Clear;
  ObjFrameDeBase.Initialiser;
  FNomsDesAnimations.Clear;
  FVitesse:= 1;
  ExTime:= 0;
  FExT:= 0;
  FCurrentAnimation:= -1;
  EnAttente:= False;
  IsSimple:= False;
end;

procedure TObjAnimations.SetVitesse(Value: single);
var
  i: integer;
begin
  if Value <> FVitesse then begin
    EnAttente:= True;
    FVitesse:= Value;
    if Animations.count > 0 then
      for i:= 0 to Animations.count - 1 do
        TObjAnimation(Animations[i]).Vitesse:= Value;
    EnAttente:= False;
    end;
end;

procedure TObjAnimations.SetCurrentAnimation(Value: integer);
begin
  if Value <> FCurrentAnimation then begin
    FCurrentAnimation:= Value;
    if Value < 0 then exit;
    if Value > Animations.count - 1 then exit;
    EnAttente:= True;
    PlaySound(nil, 0, SND_PURGE);
    TObjAnimation(Animations[FCurrentAnimation]).CurrentFrame:= 0;
    EnAttente:= False;
    end;
end;

procedure TObjAnimations.Progress(const deltaTime, newTime : Double);
var
  ExFrame, ExAnimation: integer;
  NewFrame, NewAnimation: integer;
  ResteDuree, v, d, t, CoeffDuree: Single;
  ObjAnimation: TObjAnimation;
  ObjFrame: TObjFrame;
begin
  if EnAttente then exit;
  if Animations.count < 1 then exit;
  EnAttente:= True;
  try
    ExAnimation:= FCurrentAnimation;
    ExFrame:= TObjAnimation(Animations[ExAnimation]).CurrentFrame;
    NewAnimation:= ExAnimation;
    NewFrame:= ExFrame;
    if ExTime < 0 then ExTime:= newTime;
    ResteDuree:= (newTime - ExTime) * 1000;
    ObjAnimation:= TObjAnimation(Animations[ExAnimation]);
    v:= ObjAnimation.Vitesse;
    ObjFrame:= TObjFrame(ObjAnimation.Frames[ExFrame]);
    d:= ObjFrame.Duree;
    CoeffDuree:= d;
    while ResteDuree * v > d do begin
      ResteDuree:= ResteDuree - d / v;
      NewFrame:= NewFrame + 1;
      if NewFrame > ObjAnimation.Frames.count - 1 then begin
        if ObjAnimation.Loop = True
          then NewFrame:= 0
          else begin
            NewAnimation:= ObjAnimation.Suivante;
            NewFrame:= 0;
            ObjAnimation:= TObjAnimation(Animations[NewAnimation]);
            v:= ObjAnimation.Vitesse;
            end;
        end;
      ObjFrame:= TObjFrame(ObjAnimation.Frames[NewFrame]);
      d:= ObjFrame.Duree;
      end;
    t:= ResteDuree * v / CoeffDuree;
    if (NewAnimation <> ExAnimation) or (NewFrame <> ExFrame)
      then begin
        ExTime:= newTime;
        Affiche(NewAnimation, NewFrame);
        end
      else if t-FExT > EPSILON3 then begin
        NewFrame:= NewFrame + 1;
        if NewFrame > ObjAnimation.Frames.count - 1 then begin
          if ObjAnimation.Loop = True
            then NewFrame:= 0
            else begin
              NewAnimation:= ObjAnimation.Suivante;
              NewFrame:= 0;
              end;
          end;
        if (NewAnimation <> ExAnimation) or (NewFrame <> ExFrame) then
          ChangeVisible(ExAnimation, ExFrame, NewAnimation, NewFrame, t);
        end;
  finally
    EnAttente:= False;
    end;
end;

procedure TObjAnimations.ChangeVisible(Animation1, Frame1,
                                       Animation2, Frame2: integer;
                                       t: single);
var
  ObjFrame1, ObjFrame2: TObjFrame;
  i, j: integer;
  ooo: TGLBaseSceneObject;
  ObjSimple1, ObjSimple2, ObjSimple: TObjSimple;
  lerp: single;
begin
  ObjFrame1:= TObjFrame(TObjAnimation(Animations[Animation1]).Frames[Frame1]);
  ObjFrame2:= TObjFrame(TObjAnimation(Animations[Animation2]).Frames[Frame2]);
  if t-FExT < EPSILON3 then exit;
  lerp:= (t-FExT)/(1-FExT);
  ObjSimple1:= TObjSimple.Create;
  ObjSimple:= TObjSimple.Create;
  try
    if ObjFrame2.Components.count > 0 then
      for i:= 0 to ObjFrame2.Components.count - 1 do begin
        ObjSimple2:= TObjSimple(ObjFrame2.Components[i]);
        ooo:= ObjSimple2.GLObj;
        RecupereValeurs(ooo, ObjSimple1);
        ValeursLerp(ObjSimple1, ObjSimple2, ObjSimple, lerp);
        ObjSimple.ToGLObj;
        end;
  finally
    ObjSimple1.Free;
    ObjSimple.Free;
    end;
  FExT:= t;
end;

procedure TObjAnimations.Affiche(Animation, Frame: integer);
var
  ObjAnimation: TObjAnimation;
  DoitJouerSon: boolean;
  NumSon: integer;
  ObjFrame: TObjFrame;
  i, j: integer;
begin
  FExT:= 0;
  //Si l'Animation n'existe pas :
  if Animation > Animations.count - 1 then begin
    Animation:= 0;
    Frame:= 0;
    end;
  CurrentAnimation:= Animation;
  ObjAnimation:= TObjAnimation(Animations[Animation]);
  ObjAnimation.CurrentFrame:= Frame;
  //On doit jouer le son ?
  DoitJouerSon:= False;
  NumSon:= ObjAnimation.NumFrameSon;
  if NumSon > -1 then
    if Frame = NumSon then DoitJouerSon:= True;
  //On y va :
  if DoitJouerSon = True then
    PlaySound(ObjAnimation.MemoryStreamFichierSon.Memory,
              0, SND_ASYNC or SND_MEMORY);
  ObjFrame:= TObjFrame(ObjAnimation.Frames[Frame]);
  if ObjFrame.Components.count > 0 then
    for i:= 0 to ObjFrame.Components.count - 1 do
      TObjSimple(ObjFrame.Components[i]).ToGLObj;
  ExTime:= -1;
end;

procedure TObjAnimations.MetAJourNomsDesAnimations;
var
  i: integer;
begin
  FNomsDesAnimations.clear;
  if Animations.Count > 0 then
    for i:= 0 to Animations.Count - 1 do
      FNomsDesAnimations.Add(TObjAnimation(Animations[i]).Nom);
end;

function TObjAnimations.JoueAnimationViaNom(NomAnimation: string): boolean;
var
  i, j: integer;
begin
  j:= -1;
  if Animations.Count > 0 then
    for i:= 0 to Animations.Count - 1 do
      if TObjAnimation(Animations[i]).Nom = NomAnimation then begin
        j:= i;
        break;
        end;
  if j > -1
    then Result:= True
    else j:= 0;
  if j <> FCurrentAnimation then Affiche(j, 0);
  Result:= True;
end;

function TObjAnimations.JoueAnimationViaNomVitesse(NomAnimation: string; Coeff: single): boolean;
var
  i, j: integer;
begin
  j:= -1;
  if Animations.Count > 0 then
    for i:= 0 to Animations.Count - 1 do
      if TObjAnimation(Animations[i]).Nom = NomAnimation then begin
        j:= i;
        break;
        end;
  if j > -1
    then Result:= True
    else j:= 0;
  if Coeff <> TObjAnimation(Animations[j]).FVitesse then
    TObjAnimation(Animations[j]).Vitesse:= Coeff;
  if j <> FCurrentAnimation then Affiche(j, 0);
  Result:= True;
end;

function TObjAnimations.RetourneActuel: TObjActuel;
begin
  Result.IndexAnimation:= -1;
  Result.IndexFrame:= -1;
  Result.NomAnimation:= '';
  Result.OK:= False;
  if Animations.count < 1 then exit;
  Result.IndexAnimation:= CurrentAnimation;
  if Result.IndexAnimation < 0 then exit;
  with TObjAnimation(Animations[CurrentAnimation]) do begin
    Result.IndexFrame:= CurrentFrame;
    Result.NomAnimation:= Nom;
    end;
  Result.OK:= True;
end;

function TObjAnimations.AnimationActuelle: TObjAnimation;
begin
  if Animations.Count > 0
    then Result:= TObjAnimation(Animations[CurrentAnimation])
    else result:= nil;
end;

function TObjAnimations.RetourneNomAnimationActuelle: string;
begin
  if Animations.Count > 0
    then Result:= TObjAnimation(Animations[CurrentAnimation]).Nom
    else result:= '';
end;

procedure TObjAnimations.AjouteAnimation(var NewNom: string);
var
  i, j: integer;
  BaseNom: string;
  NewNomOK: Boolean;
  ObjAnimation: TObjAnimation;
begin
  BaseNom:= NewNom;
  j:= 1;
  if Animations.Count > 0 then begin
    while NewNomOK = False do begin
      NewNomOK:= True;
      for i:= 0 to Animations.Count - 1 do
        if TObjAnimation(Animations[i]).Nom = NewNom then begin
          NewNom:= BaseNom + inttostr(j);
          j:= j + 1;
          NewNomOK:= False;
          Break;
          end;
      end;
    end;
  ObjAnimation:= TObjAnimation.Create;
  ObjAnimation.Nom:= NewNom;
  ObjAnimation.Vitesse:= Vitesse;
  Animations.Add(ObjAnimation);
  FCurrentAnimation:= Animations.Count - 1;
end;

procedure TObjAnimations.SupprimerAnimation(IndexAnimationASupprimer: integer);
var
  i: integer;
  ObjAnimation: TObjAnimation;
begin
  if Animations.Count > 0 then begin
    for i:= 0 to Animations.Count - 1 do begin
      ObjAnimation:= TObjAnimation(Animations[i]);
      if ObjAnimation.Suivante = IndexAnimationASupprimer then ObjAnimation.Suivante:= 0;
      end;
    TObjAnimation(Animations[IndexAnimationASupprimer]).free;
    Animations.Delete(IndexAnimationASupprimer);
    FCurrentAnimation:= 0;
    TObjAnimation(Animations[CurrentAnimation]).CurrentFrame:= 0;
    end;
end;

procedure TObjAnimations.Deplace(IndexAnimationADeplacer, IndexFrameADeplacer,
                                 IndexAnimationCible, IndexFrameCible: integer);
var
  i, n: integer;
  ObjFrame1, ObjFrame2: TObjFrame;
  ObjSimple1, ObjSimple2: TObjSimple;
  ObjAnimation1, ObjAnimation2: TObjAnimation;
begin
  if IndexFrameCible = -1
    //Si on dplace une Animation :
    then Animations.Move(IndexAnimationADeplacer, IndexAnimationCible)
    //si on dplace une Frame :
    else begin
      if IndexAnimationADeplacer = IndexAnimationCible
        //dans la mme animation :
        then begin
          ObjAnimation1:= TObjAnimation(Animations[IndexAnimationADeplacer]);
          if ObjAnimation1.NumFrameSon = IndexFrameADeplacer then
            ObjAnimation1.NumFrameSon:= IndexFrameCible;
          ObjAnimation1.Frames.Move(IndexFrameADeplacer, IndexFrameCible);
          end
        //dans une autre :
        else begin
          ObjAnimation1:= TObjAnimation(Animations[IndexAnimationADeplacer]);
          ObjAnimation2:= TObjAnimation(Animations[IndexAnimationCible]);
          if ObjAnimation1.NumFrameSon = IndexFrameADeplacer then begin
            ObjAnimation2.NumFrameSon:= IndexFrameCible;
            ObjAnimation2.MemoryStreamFichierSon.clear;
            ObjAnimation1.MemoryStreamFichierSon.Position:= 0;
            ObjAnimation2.MemoryStreamFichierSon.CopyFrom(ObjAnimation1.MemoryStreamFichierSon, ObjAnimation1.MemoryStreamFichierSon.Size);

            ObjAnimation1.NumFrameSon:= -1;
            ObjAnimation1.MemoryStreamFichierSon.clear;
            end;
          ObjFrame1:= TObjFrame(ObjAnimation1.Frames[IndexFrameADeplacer]);
          ObjFrame2:= TObjFrame.Create;
          for i:= 0 to ObjFrame1.Components.Count - 1 do begin
            ObjSimple1:= TObjSimple(ObjFrame1.Components[i]);
            ObjSimple2:= TObjSimple.create;
            RecopieObjSimple(ObjSimple1, ObjSimple2);
            ObjFrame2.Components.Add(ObjSimple2);
            end;
          ////////////////////////ObjFrame2.FromGLObj;
          ObjAnimation2.Frames.Add(ObjFrame2);
          n:= ObjAnimation2.Frames.count;
          ObjAnimation2.Frames.Move(n - 1, IndexFrameCible);
          ObjAnimation1.SupprimerFrame(IndexFrameADeplacer);
          end;
      end;
end;

procedure TObjAnimations.FromGLObj;
var
  i, j, k: integer;
  ObjAnimation: TObjAnimation;
  ObjFrame: TObjFrame;
begin
  if Animations.Count > 0 then
    for i:= 0 to Animations.Count - 1 do begin
      ObjAnimation:= TObjAnimation(Animations[i]);
      if ObjAnimation.Frames.count > 0 then
        for j:= 0 to ObjAnimation.Frames.count - 1 do begin
          ObjFrame:= TObjFrame(ObjAnimation.Frames[j]);
          if ObjFrame.Components.Count > 0 then
            for k:= 0 to ObjFrame.Components.Count - 1 do
              TObjSimple(ObjFrame.Components[k]).FromGLObj;
          end;
      end;
end;

procedure TObjAnimations.ToGLObj;
var
  i, j, k: integer;
  ObjAnimation: TObjAnimation;
  ObjFrame: TObjFrame;
begin
  if Animations.Count > 0 then
    for i:= 0 to Animations.Count - 1 do begin
      ObjAnimation:= TObjAnimation(Animations[i]);
      if ObjAnimation.Frames.count > 0 then
        for j:= 0 to ObjAnimation.Frames.count - 1 do begin
          ObjFrame:= TObjFrame(ObjAnimation.Frames[j]);
          if ObjFrame.Components.Count > 0 then
            for k:= 0 to ObjFrame.Components.Count - 1 do
              TObjSimple(ObjFrame.Components[k]).ToGLObj;
          end;
      end;
end;

procedure TObjAnimations.MetEnPlaceLesGLObj;

    procedure Recursion(ObjSimple: TObjSimple; AGLObj: TGLBaseSceneObject);
    var
      i: integer;
      NewObjSimple: TObjSimple;
    begin
      with ObjSimple do begin
        GLObj:= AGLObj;
        if Fils.Count > 0 then
          for i:= 0 to Fils.Count - 1 do begin
            NewObjSimple:= Fils[i];
            Recursion(NewObjSimple, AGLObj.Children[i]);
            end;
        end;

    end;

var
  i, j, k: integer;
  ObjAnimation: TObjAnimation;
  ObjFrame: TObjFrame;
  SimpleEx: boolean;
begin
  if Animations.Count < 1 then exit;
  if IsSimple then Compliquer;

  if ObjFrameDeBase.Components.Count > 0 then
    for k:= 0 to ObjFrameDeBase.Components.Count - 1 do
      Recursion(TObjSimple(ObjFrameDeBase.Components[k]), GLParent.Children[k]);

  for i:= 0 to Animations.Count - 1 do begin
    ObjAnimation:= TObjAnimation(Animations[i]);
    if ObjAnimation.Frames.Count > 0 then
      for j:= 0 to ObjAnimation.Frames.Count - 1 do begin
        ObjFrame:= TObjFrame(ObjAnimation.Frames[j]);
        if ObjFrame.Components.Count > 0 then
          for k:= 0 to ObjFrame.Components.Count - 1 do
            Recursion(TObjSimple(ObjFrame.Components[k]), GLParent.Children[k]);
        end;
    end;
end;



procedure TObjAnimations.Simplifier(PourEnregistrer: boolean);

    function Recursion(ObjSimple1: TObjSimple; var ObjSimple2: TObjSimple): boolean;
    var
      i: integer;
      NewObjSimple: TObjSimple;
    begin
      Result:= False;
      with ObjSimple2 do begin
        if Fils.Count > 0 then
          for i:= Fils.Count - 1 downto 0 do begin
            NewObjSimple:= Fils[i];
            if Recursion(ObjSimple1.Fils[i], NewObjSimple)then begin
              NewObjSimple.free;
              Fils.Delete(i);
              end;
            end;
        if Fils.Count > 0 then exit;
        if VectorEquals(ObjSimple1.ObjValeurs.Position, ObjValeurs.Position) = False then exit;
        if VectorEquals(ObjSimple1.ObjValeurs.Direction, ObjValeurs.Direction) = False then exit;
        if VectorEquals(ObjSimple1.ObjValeurs.Up, ObjValeurs.Up) = False then exit;
        if VectorEquals(ObjSimple1.ObjValeurs.Scale, ObjValeurs.Scale) = False then exit;
        if VectorEquals(ObjSimple1.ObjValeurs.Couleur, ObjValeurs.Couleur) = False then exit;
        if ObjSimple1.ObjValeurs.Transparence <> ObjValeurs.Transparence then exit;
        end;
      Result:= True;
    end;

var
  i, j, k: integer;
  ObjAnimation: TObjAnimation;
  ObjFrame1, ObjFrame2: TObjFrame;
  ObjSimple1, ObjSimple2: TObjSimple;
begin
  if IsSimple then exit;
  if Animations.Count > 0 then
    for i:= 0 to Animations.Count - 1 do begin
      ObjAnimation:= TObjAnimation(Animations[i]);
      if ObjAnimation.Frames.count > 1 then
        for j:= ObjAnimation.Frames.count - 1 downto 1 do begin
          ObjFrame1:= TObjFrame(ObjAnimation.Frames[j - 1]);
          ObjFrame2:= TObjFrame(ObjAnimation.Frames[j]);
          for k:= ObjFrame1.Components.Count - 1 downto 0 do begin
            ObjSimple1:= TObjSimple(ObjFrame1.Components[k]);
            ObjSimple2:= TObjSimple(ObjFrame2.Components[k]);
            if Recursion(ObjSimple1, ObjSimple2) then begin
              ObjSimple2.free;
              ObjFrame2.Components.Delete(k);
              end;
            end;
          end;
      if PourEnregistrer then begin
        ObjFrame2:= TObjFrame(ObjAnimation.Frames[0]);
        for k:= ObjFrameDeBase.Components.Count - 1 downto 0 do begin
          ObjSimple1:= TObjSimple(ObjFrameDeBase.Components[k]);
          ObjSimple2:= TObjSimple(ObjFrame2.Components[k]);
          if Recursion(ObjSimple1, ObjSimple2) then begin
            ObjSimple2.free;
            ObjFrame2.Components.Delete(k);
            end;
          end;
        end;
      end;
  IsSimple:= True;
end;

procedure TObjAnimations.Compliquer;

    procedure Recursion(ObjSimple1: TObjSimple; var ObjSimple2: TObjSimple);
    var
      i, j: integer;
      NewObjSimple1, NewObjSimple2: TObjSimple;
    begin
      for i:= 0 to ObjSimple1.Fils.Count - 1 do begin
        NewObjSimple1:= ObjSimple1.Fils[i];
        if ObjSimple2.Fils.count < i + 1
          then j:= -1
          else j:= TObjSimple(ObjSimple2.Fils[i]).GLObj.Index;
        if NewObjSimple1.GLObj.Index <> j
          then begin
            NewObjSimple2:= TObjSimple.create;
            RecopieObjSimple(NewObjSimple1, NewObjSimple2);
            ObjSimple2.Fils.Insert(i, NewObjSimple2);
            end
          else begin
            NewObjSimple2:= TObjSimple(ObjSimple2.Fils[i]);
            Recursion(NewObjSimple1, NewObjSimple2);
            end;
        end;
    end;

var
  i, j, k, p: integer;
  ObjAnimation: TObjAnimation;
  ObjFrame1, ObjFrame2: TObjFrame;
  ObjSimple1, ObjSimple2: TObjSimple;
begin         
  if IsSimple = False then exit;
  if Animations.Count > 0 then
    for i:= 0 to Animations.Count - 1 do begin
      ObjAnimation:= TObjAnimation(Animations[i]);

      ObjFrame2:= TObjFrame(ObjAnimation.Frames[0]);
      for k:= 0 to ObjFrameDeBase.Components.Count - 1 do begin
        ObjSimple1:= TObjSimple(ObjFrameDeBase.Components[k]);
        if ObjFrame2.Components.count < k + 1
          then p:= -1
          else p:= TObjSimple(ObjFrame2.Components[k]).GLObj.Index;
        if ObjSimple1.GLObj.Index <> p
          then begin
            ObjSimple2:= TObjSimple.create;
            RecopieObjSimple(ObjSimple1, ObjSimple2);
            ObjFrame2.Components.Insert(k, ObjSimple2);
            end
          else begin
            ObjSimple2:= TObjSimple(ObjFrame2.Components[k]);
            Recursion(ObjSimple1, ObjSimple2);
            end;
        end;


      if ObjAnimation.Frames.count > 1 then
        for j:= 1 to ObjAnimation.Frames.count - 1 do begin
          ObjFrame1:= TObjFrame(ObjAnimation.Frames[j - 1]);
          ObjFrame2:= TObjFrame(ObjAnimation.Frames[j]);
          for k:= 0 to ObjFrame1.Components.Count - 1 do begin
            ObjSimple1:= TObjSimple(ObjFrame1.Components[k]);
            if ObjFrame2.Components.count < k + 1
              then p:= -1
              else p:= TObjSimple(ObjFrame2.Components[k]).GLObj.Index;
            if ObjSimple1.GLObj.Index <> p
              then begin
                ObjSimple2:= TObjSimple.create;
                RecopieObjSimple(ObjSimple1, ObjSimple2);
                ObjFrame2.Components.Insert(k, ObjSimple2);
                end
              else begin
                ObjSimple2:= TObjSimple(ObjFrame2.Components[k]);
                Recursion(ObjSimple1, ObjSimple2);
                end;
            end;
          end;
      end;
  IsSimple:= False;
end;

procedure TObjAnimations.SupprimeComponent(Component: TGLBaseSceneObject);
var
  i: integer;
  SimpleEx: boolean;
begin
  if Animations.count = 0 then exit;
  SimpleEx:= IsSimple;
  Compliquer;
  if Animations.Count > 0 then
    for i:= 0 to Animations.Count - 1 do
      TObjAnimation(Animations[i]).SupprimeComponent(Component);
  ObjFrameDeBase.SupprimeComponent(Component);

  if SimpleEx then Simplifier(False);
end;


procedure TObjAnimations.SaveToStream(Stream: TStream);
var
  i, j, k, l: integer;
  s: Longint;
  ObjAnimation: TObjAnimation;
  ObjFrame: TObjFrame;
  ObjSimple: TObjSimple;
  SimpleEx: boolean;
  ComponentStream: TMemoryStream;
begin
  if Animations.Count < 1 then exit;
  SimpleEx:= IsSimple;
  Compliquer;
  Simplifier(True);
  Stream.Write(Animations.Count, SizeOf(integer));
  ComponentStream:= TMemoryStream.create;
  try
    for i:= 0 to Animations.Count - 1 do begin
      ObjAnimation:= TObjAnimation(Animations[i]);
      with ObjAnimation do begin
        Stream.Write(Nom, SizeOf(TNameString));
        Stream.Write(Loop, SizeOf(boolean));
        Stream.Write(Suivante, SizeOf(integer));
        Stream.Write(NumFrameSon, SizeOf(integer));
        Stream.Write(Vitesse, SizeOf(single));
        end;
      if ObjAnimation.NumFrameSon > -1 then begin
        s:= ObjAnimation.MemoryStreamFichierSon.Size;
        Stream.Write(s, SizeOf(s));
        ObjAnimation.MemoryStreamFichierSon.Position:= 0;
        Stream.CopyFrom(ObjAnimation.MemoryStreamFichierSon, s);
        end;
      Stream.Write(ObjAnimation.Frames.Count, SizeOf(integer));
      if ObjAnimation.Frames.Count > 0 then
        for j:= 0 to ObjAnimation.Frames.Count - 1 do begin
          ObjFrame:= TObjFrame(ObjAnimation.Frames[j]);
          with ObjFrame do begin
            Stream.Write(Duree, SizeOf(integer));
            Stream.Write(Components.Count, SizeOf(integer));
            end;
          if ObjFrame.Components.Count > 0 then
            for k:= 0 to ObjFrame.Components.Count - 1 do begin
              ObjSimple:= TObjSimple(ObjFrame.Components[k]);
              l:= ObjSimple.GLObj.Index;
              Stream.Write(l, SizeOf(integer));
  /////////////////////////////////////////////////////////////showmessage('ici');
              ComponentStream.Clear;
              ComponentStream.Position:= 0;
              ObjSimple.ToGLObj;
              Application.ProcessMessages;
              ComponentStream.WriteComponent(TComponent(ObjSimple.GLObj));

              s:= ComponentStream.Size;
              Stream.Write(s, SizeOf(s));
              ComponentStream.Position:= 0;
              Stream.CopyFrom(ComponentStream, s);
              end;
          end;
      end;
  finally
    ComponentStream.Free;
    end;
  Compliquer;
  Affiche(0, 0);
  if SimpleEx then Simplifier(False);
end;

procedure TObjAnimations.SaveToXML(XML: TGmXML);

    procedure Recursion(ObjSimple: TObjSimple);
    var
      i: integer;
      NewObjSimple: TObjSimple;
    begin
      with ObjSimple do begin
        XML.Nodes.AddOpenTag(GLObj.Name);
        XML.Nodes.AddOpenTag('Position').AsString:= Vector2String(ObjValeurs.Position); XML.Nodes.AddCloseTag;
        XML.Nodes.AddOpenTag('Direction').AsString:= Vector2String(ObjValeurs.Direction); XML.Nodes.AddCloseTag;
        XML.Nodes.AddOpenTag('Up').AsString:= Vector2String(ObjValeurs.Up); XML.Nodes.AddCloseTag;
        XML.Nodes.AddOpenTag('Scale').AsString:= Vector2String(ObjValeurs.Scale); XML.Nodes.AddCloseTag;
        XML.Nodes.AddOpenTag('Couleur').AsString:= Vector2String(ObjValeurs.Couleur); XML.Nodes.AddCloseTag;
        XML.Nodes.AddOpenTag('Transparence').AsFloat:= ObjValeurs.Transparence; XML.Nodes.AddCloseTag;

        if Fils.Count > 0 then
          for i:= 0 to Fils.Count - 1 do begin
            NewObjSimple:= Fils[i];
            Recursion(NewObjSimple);
            end;

        XML.Nodes.AddCloseTag;
        end;
    end;

var
  i, j, k: integer;
  ObjAnimation: TObjAnimation;
  ObjFrame: TObjFrame;
  SimpleEx: boolean;

  ss1: string;
  Buffer: array[0..31] of Char;
  Text: array[0..63] of Char;
  b, t: char;
begin
  if Animations.Count < 1 then exit;
  SimpleEx:= IsSimple;
  Compliquer;
  Simplifier(True);

  XML.Nodes.AddOpenTag('Animations');
  XML.Nodes.AddOpenTag('AnimationsCount').AsInteger:= Animations.Count; XML.Nodes.AddCloseTag;

  for i:= 0 to Animations.Count - 1 do begin
    ObjAnimation:= TObjAnimation(Animations[i]);
    XML.Nodes.AddOpenTag('Animation' + inttostr(i));
    with ObjAnimation do begin
      XML.Nodes.AddOpenTag('Nom').AsString:= Nom; XML.Nodes.AddCloseTag;
      XML.Nodes.AddOpenTag('Loop').AsBoolean:= Loop; XML.Nodes.AddCloseTag;
      XML.Nodes.AddOpenTag('Suivante').AsInteger:= Suivante; XML.Nodes.AddCloseTag;
      XML.Nodes.AddOpenTag('NumFrameSon').AsInteger:= NumFrameSon; XML.Nodes.AddCloseTag;
      XML.Nodes.AddOpenTag('Vitesse').AsFloat:= Vitesse; XML.Nodes.AddCloseTag;
      end;

    if ObjAnimation.NumFrameSon > -1 then begin
      ObjAnimation.MemoryStreamFichierSon.Position:= 0;
      ss1:= '';
      while ObjAnimation.MemoryStreamFichierSon.Position < ObjAnimation.MemoryStreamFichierSon.Size do begin
        FillChar(Text, SizeOf(Text), 0);
        FillChar(Buffer, Sizeof(Buffer), 0);
        j:= ObjAnimation.MemoryStreamFichierSon.Read(Buffer, Sizeof(Buffer));
        BinToHex(Buffer, Text, j);
        for k:= 1 to 2*j do ss1:= ss1 + Text[k-1];
        end;
      XML.Nodes.AddOpenTag('FichierSon').AsString:= ss1; XML.Nodes.AddCloseTag;
      end;

    XML.Nodes.AddOpenTag('FramesCount').AsInteger:= ObjAnimation.Frames.Count; XML.Nodes.AddCloseTag;
    if ObjAnimation.Frames.Count > 0 then
      for j:= 0 to ObjAnimation.Frames.Count - 1 do begin
        ObjFrame:= TObjFrame(ObjAnimation.Frames[j]);
        XML.Nodes.AddOpenTag('Frame' + inttostr(j));
        with ObjFrame do
          XML.Nodes.AddOpenTag('Duree').AsInteger:= Duree; XML.Nodes.AddCloseTag;

        XML.Nodes.AddOpenTag('Components');
        if ObjFrame.Components.Count > 0 then
          for k:= 0 to ObjFrame.Components.Count - 1 do
            Recursion(TObjSimple(ObjFrame.Components[k]));
        XML.Nodes.AddCloseTag;

        XML.Nodes.AddCloseTag;
        end;
    XML.Nodes.AddCloseTag;
    end;
  XML.Nodes.AddCloseTag;

  Compliquer;
  if SimpleEx then Simplifier(False);
end;













procedure TObjAnimations.gloups;

    procedure Recursion(ObjSimple: TObjSimple; var a, b: string);
    var
      i: integer;
      c: string;
      NewObjSimple: TObjSimple;
    begin
      with ObjSimple do begin
        if GLObj <> nil
          then a:= a + #13 + b + GLObj.Name
          else a:= a + #13 + b + 'nil';
        if Fils.Count > 0 then
          for i:= 0 to Fils.Count - 1 do begin
            NewObjSimple:= Fils[i];
            c:= b + '>';
            Recursion(NewObjSimple, a, c);
            end;
        end;
    end;

var
  i, j, k: integer;
  a, b: string;

  ObjAnimation: TObjAnimation;
  ObjFrame: TObjFrame;
begin
  if Animations.Count < 1 then exit;

  a:= 'gloups :' + #13;
  for i:= 0 to Animations.Count - 1 do begin
    ObjAnimation:= TObjAnimation(Animations[i]);
    if ObjAnimation.Frames.Count > 0 then
      for j:= 0 to ObjAnimation.Frames.Count - 1 do begin
        ObjFrame:= TObjFrame(ObjAnimation.Frames[j]);

        if ObjFrame.Components.Count > 0 then
          for k:= 0 to ObjFrame.Components.Count - 1 do
            Recursion(TObjSimple(ObjFrame.Components[k]), a, b);


        end;
    end;
  showmessage(a);
end;

procedure TObjAnimations.LoadFromXML(XML: TGmXML);

    procedure Recursion(ObjFrame: TObjFrame; ObjSimple: TObjSimple; GmXmlNode: TGmXmlNode);
    var
      i: integer;
      a: string;
      NewObjSimple: TObjSimple;
      AObjValeurs: TObjValeurs;
    begin
      if GmXmlNode.IsLeafNode = False
        then begin
          if ObjSimple <> nil
            then NewObjSimple:= TrouveObjSimpleParGLObjName(GmXmlNode.Name, ObjSimple.Fils, False, 0)
            else NewObjSimple:= TrouveObjSimpleParGLObjName(GmXmlNode.Name, ObjFrame.Components, False, 0);
          if GmXmlNode.Children.Count > 0 then
            for i:= 0 to GmXmlNode.Children.Count - 1 do
              Recursion(ObjFrame, NewObjSimple, GmXmlNode.Children[i]);
          end
        else begin
          if ObjSimple <> nil then begin
            AObjValeurs:= ObjSimple.ObjValeurs;
            a:= GmXmlNode.Name;
            if a = 'Position' then AObjValeurs.Position:= String2Vector(GmXmlNode.AsString);
            if a = 'Direction' then AObjValeurs.Direction:= String2Vector(GmXmlNode.AsString);
            if a = 'Up' then AObjValeurs.Up:= String2Vector(GmXmlNode.AsString);
            if a = 'Scale' then AObjValeurs.Scale:= String2Vector(GmXmlNode.AsString);
            if a = 'Couleur' then AObjValeurs.Couleur:= String2Vector(GmXmlNode.AsString);
            if a = 'Transparence' then AObjValeurs.Transparence:= GmXmlNode.AsFloat;
            ObjSimple.ObjValeurs:= AObjValeurs;
            end;
          end;
    end;

var
  i, j, k, p, NbA, NbF: integer;
  a: string;
  ObjAnimation: TObjAnimation;
  ObjFrame, ObjFrameACopier: TObjFrame;
  ObjSimple: TObjSimple;

  GmXmlNode: TGmXmlNode;
  ss1: string;
  Buffer: array[0..31] of Char;
  Text: array[0..63] of Char;
  FrameDeBaseExiste: boolean;
begin
  Initialiser;
  EnAttente:= True;

  with GLParent do
    if Count > 0 then
      for i:= 0 to Count - 1 do
        ObjFrameDeBase.AjouteComponent(Children[i]);

  NbA:= XML.GetNodeByPath('Animations\AnimationsCount').AsInteger;
  for i:= 0 to NbA - 1 do begin
    a:= 'Animations\Animation' + inttostr(i);
    ObjAnimation:= TObjAnimation.Create;
    Animations.Add(ObjAnimation);
    with ObjAnimation do begin
      Nom:= XML.GetNodeByPath(a + '\Nom').AsString;
      Loop:= XML.GetNodeByPath(a + '\Loop').AsBoolean;
      Suivante:= XML.GetNodeByPath(a + '\Suivante').AsInteger;
      NumFrameSon:= XML.GetNodeByPath(a + '\NumFrameSon').AsInteger;
      Vitesse:= XML.GetNodeByPath(a + '\Vitesse').AsFloat;
      end;


    if ObjAnimation.NumFrameSon > -1 then begin
      ObjAnimation.MemoryStreamFichierSon:= TMemoryStream.create;
      ss1:= XML.GetNodeByPath(a + '\FichierSon').AsString;

      while length(ss1) > 0 do begin
        p:= length(ss1);
        if p > 64 then p:= 64;
        FillChar(Text, SizeOf(Text), 0);
        FillChar(Buffer, Sizeof(Buffer), 0);
        for k:= 1 to p do Text[k-1]:= ss1[k];
        Delete(ss1, 1, p);
        k:= HexToBin(Text, Buffer, Sizeof(Buffer));
        ObjAnimation.MemoryStreamFichierSon.Write(Buffer, k);
        end;
      ObjAnimation.MemoryStreamFichierSon.Position:= 0;
      end;

      

    NbF:= XML.GetNodeByPath(a + '\FramesCount').AsInteger;
    if NbF > 0 then
      for j:= 0 to NbF - 1 do begin
        a:= 'Animations\Animation' + inttostr(i) + '\Frame' + inttostr(j);
        ObjFrame:= TObjFrame.Create;
        ObjAnimation.Frames.Add(ObjFrame);
        ObjFrame.Duree:= XML.GetNodeByPath(a + '\Duree').AsInteger;

        if j = 0
          then ObjFrameACopier:= ObjFrameDeBase
          else ObjFrameACopier:= ObjAnimation.Frames[j-1];
        for k:= 0 to ObjFrameACopier.Components.Count - 1 do begin
          ObjSimple:= TObjSimple.Create;
          ObjFrame.Components.Add(ObjSimple);
          RecopieObjSimple(TObjSimple(ObjFrameACopier.Components[k]), ObjSimple);
          end;
        GmXmlNode:= XML.GetNodeByPath(a + '\Components');
        if GmXmlNode <> nil then if GmXmlNode.Children.Count > 0 then
          for k:= 0 to GmXmlNode.Children.Count - 1 do
            Recursion(ObjFrame, nil, GmXmlNode.Children[k]);

        end;
//???????    ObjAnimation.Vitesse:= v;
    end;

  MetEnPlaceLesGLObj;
  simplifier(False);
  Compliquer;

  Affiche(0, 0);
  EnAttente:= False;
end;


procedure ComponentStream2ObjSimple(ComponentStream: TMemoryStream; var ObjSimple: TObjSimple);
var
  ooo: TGLBaseSceneObject;
begin
  ooo:= TGLBaseSceneObject.Create(nil);
  try
    ComponentStream.Position:= 0;
    ooo:= TGLBaseSceneObject(ComponentStream.ReadComponent(nil));
    RecupereValeurs(ooo, ObjSimple);
  finally
    ooo.Free;
    end;
end;

procedure TObjAnimations.LoadFromStream(Stream: TStream);
var
  i, j, k, d, NbA, NbF, NbC: integer;
  s: Longint;
  a: TNameString;
  b: boolean;
  v: single;
  ObjAnimation: TObjAnimation;
  ObjFrame, ObjFrameACopier: TObjFrame;
  ObjSimple: TObjSimple;
  ComponentStream: TMemoryStream;
begin
  Initialiser;
  EnAttente:= True;

  with GLParent do
    if Count > 0 then
      for i:= 0 to Count - 1 do
        ObjFrameDeBase.AjouteComponent(Children[i]);

  ComponentStream:= TMemoryStream.Create;
  try
    Stream.Read(NbA, SizeOf(integer));
    for i:= 0 to NbA - 1 do begin
      ObjAnimation:= TObjAnimation.Create;
      Animations.Add(ObjAnimation);
      with ObjAnimation do begin
        Stream.Read(a, SizeOf(TNameString));
        Nom:= a;
        Stream.Read(b, SizeOf(boolean));
        Loop:= b;
        Stream.Read(d, SizeOf(integer));
        Suivante:= d;
        Stream.Read(d, SizeOf(integer));
        NumFrameSon:= d;
        Stream.Read(v, SizeOf(single));
        end;
      if ObjAnimation.NumFrameSon > -1 then begin
        Stream.Read(s, SizeOf(s));
        ObjAnimation.MemoryStreamFichierSon:= TMemoryStream.create;
        ObjAnimation.MemoryStreamFichierSon.CopyFrom(Stream, s);
        end;
      Stream.Read(NbF, SizeOf(integer));
      if NbF > 0 then
        for j:= 0 to NbF - 1 do begin
          ObjFrame:= TObjFrame.Create;
          ObjAnimation.Frames.Add(ObjFrame);
          with ObjFrame do begin
            Stream.Read(d, SizeOf(integer));
            Duree:= d;
            Stream.Read(d, SizeOf(integer));
            NbC:= d;
            end;

          if j = 0
            then begin
              ObjFrameACopier:= ObjFrameDeBase;
              for k:= 0 to ObjFrameACopier.Components.Count - 1 do begin
                ObjSimple:= TObjSimple.Create;
                ObjFrame.Components.Add(ObjSimple);
                RecopieObjSimple(TObjSimple(ObjFrameACopier.Components[k]), ObjSimple);
                end;
              if NbC > 0 then
                for k:= 0 to NbC - 1 do begin
                  ObjSimple:= TObjSimple(ObjFrame.Components[k]);
                  Stream.Read(d, SizeOf(integer));
                  Stream.Read(s, SizeOf(s));
                  ComponentStream.Clear;
                  ComponentStream.Position:= 0;
                  ComponentStream.CopyFrom(Stream, s);
                  ComponentStream2ObjSimple(ComponentStream, ObjSimple);
                  end;
              end
            else begin
              ObjFrameACopier:= ObjAnimation.Frames[j-1];
              for k:= 0 to ObjFrameACopier.Components.Count - 1 do begin
                ObjSimple:= TObjSimple.Create;
                ObjFrame.Components.Add(ObjSimple);
                RecopieObjSimple(TObjSimple(ObjFrameACopier.Components[k]), ObjSimple);
                end;
              if NbC > 0 then
                for k:= 0 to NbC - 1 do begin
                  Stream.Read(d, SizeOf(integer));
                  ObjSimple:= TObjSimple(ObjFrame.Components[d]);
                  Stream.Read(s, SizeOf(s));
                  ComponentStream.Clear;
                  ComponentStream.Position:= 0;
                  ComponentStream.CopyFrom(Stream, s);
                  ComponentStream2ObjSimple(ComponentStream, ObjSimple);
                  end;
              end;
          end;
      ObjAnimation.Vitesse:= v;
      end;
  finally
    ComponentStream.Free;
    end;

  MetEnPlaceLesGLObj;
  simplifier(False);
  Compliquer;

  Affiche(0, 0);
  EnAttente:= False;
end;

// ------------------
// ------------------ TObjActor ------------------
// ------------------
constructor TObjActor.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FObjAnimations:= TObjAnimations.create;
  FObjAnimations.GLParent:= TGLBaseSceneObject(self);
  AlignementVertical:= AuCentre;
  Initialiser;
end;

procedure TObjActor.Initialiser;
begin
  FObjAnimations.EnAttente:= True;
  if Count > 0 then DeleteChildren;
  FObjAnimations.Initialiser;
  FCoeffScale:= 1;
  FObjAnimations.EnAttente:= False;
end;

destructor TObjActor.Destroy;
begin
  Initialiser;
  FObjAnimations.EnAttente:= True;
  FObjAnimations.free;
  inherited Destroy;
end;

procedure TObjActor.SetCoeffScale(Value: single);
begin
  if Value <> FCoeffScale then begin
    Scale.AsVector:= PointMake(Value, Value, Value);
    FCoeffScale:= Value;
    end;
end;

procedure TObjActor.CalculeMinMax;
var
  box1: TAABB;
  v: TAffineVector;
begin
  box1:=AxisAlignedBoundingBoxUnscaled;
  if box1.min[2] >= box1.max[2] then exit;
  AABBTransform(box1, AbsoluteMatrix);
  v:= VectorSubtract(box1.max, box1.min);
  ScaleVector(v, 9/10);
  MakeVector(minmax, v);
end;

procedure TObjActor.Progress(const deltaTime, newTime : Double);
var
  v: TVector;
begin
  if FObjAnimations.EnAttente then exit;
  FObjAnimations.Progress(deltaTime, newTime);
  if Encadrement <> nil then begin
    CalculeMinMax;
    Encadrement.Scale.AsVector:= minmax;
    end;
end;

procedure TObjActor.LoadFromFile(const FileName: String; AvecPBar: boolean);
var
  FileStream: TFileStream;
  i, j: integer;
  MemoryStream: TMemoryStream;
  s: Longint;
  Component: TComponent;
  ScreenCursorEx: TCursor;
begin
  ScreenCursorEx:= Screen.Cursor;
  Screen.Cursor:= crHourGlass;
  try
    Initialiser;
    FObjAnimations.EnAttente:= True;
    OuvreFichier(self, nil, nil, False, FileName, self, FObjAnimations, AvecPBar);
    //Application.ProcessMessages;
    CalculeMinMax;
  finally
    FObjAnimations.EnAttente:= False;
    Screen.Cursor:= ScreenCursorEx;
    end;
end;

end.

