(******************************************************************************
Copyright (C) 2003 Pascal Peter

http://pascal.peter.free.fr/
pascal.peter@free.fr

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
modifier conformment aux dispositions de la Licence Publique Gnrale GNU,
telle que publie par la Free Software Foundation ; version 2 de la licence,
ou encore toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
voir la Licence Publique Gnrale GNU.

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme
temps que ce programme ; si ce n'est pas le cas,
crivez  la Free Software Foundation Inc.,
675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
-------------------------------------------------------------------------------

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************)

unit glMyFreeForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,

  GLScene, GLMisc, GLObjects, GLBehaviours,
  GLTexture, VectorGeometry, JPeg,

  GLVectorFileObjects,

  GLFile3DS, GLFileObj, GLFileSTL,
  
  VectorLists,
  MeshUtils,
  PersistentClasses;


type

  TMyFreeForm = class(TGLFreeForm)
    private
      FNomFichier: string;
    protected
    public
      min, max: TAffineVector;
      NomFichierComplet: string;

      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;

      procedure Initialiser;
      procedure MetAJourMinMax;
      procedure LoadFromFile(const filename : String); override;
      procedure Stripify;
    published
      property NomFichier: string read FNomFichier write FNomFichier stored True;
    end;

var
  IDontFindTheFile, MustBeInSameDir: string;

implementation

constructor TMyFreeForm.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);

  Visible:= True;
  //AutoCentering:=[macCenterX, macCenterY];
  UseMeshMaterials:= True;
end;


destructor TMyFreeForm.Destroy;
begin
  Initialiser;
  DeleteChildren;
  inherited Destroy;
end;


procedure TMyFreeForm.Initialiser;
begin
  MeshObjects.Clear;
  StructureChanged;
  MetAJourMinMax;
end;

procedure TMyFreeForm.MetAJourMinMax;
begin
  GetExtents(min, max);
end;


procedure TMyFreeForm.LoadFromFile(const filename : String);
var
  i: integer;
  a: string;
begin
  if FileExists(filename) = False then begin
    showmessage(IDontFindTheFile + ' : ' + filename + #13 + #13 + MustBeInSameDir);
    exit;
    end;
  Initialiser;
//  MaterialLibrary.Materials.Clear;
  a:= ChangeFileExt(FileName, '.gll');
  if FileExists(a) then MaterialLibrary.LoadFromFile(a);

  if ExtractFileExt(filename) = '.3ds' then begin
    //Material.Texture.disabled:= False;
    PitchAngle:= 90;
    //Stripify;
    end
  else if ExtractFileExt(filename) = '.obj' then begin
    //Material.Texture.disabled:= False;
    //MaterialLibrary:= nil;
    //Stripify;
    end
  else begin
    //Material.Texture.disabled:= False;
    //MaterialLibrary:= nil;
    //Stripify;
    end;

  //try
    inherited LoadFromFile(filename);
    Material.Texture.disabled:= False;
    NomFichierComplet:= filename;
    NomFichier:= ExtractFileName(filename);
//    if Material.Texture.disabled then Stripify;
  //except end;

  MetAJourMinMax;
end;


procedure TMyFreeForm.Stripify;
var
  i: Integer;
  list, normals: TAffineVectorList;
  indices: TIntegerList;
  strips: TPersistentObjectList;
  obj: TMeshObject;
  fg: TFGVertexIndexList;
  perfStart, perfStop, perfFreq: Int64;
begin
  list:= MeshObjects.ExtractTriangles;
  //QueryPerformanceCounter(perfStart);
  indices:= BuildVectorCountOptimizedIndices(list);
  RemapAndCleanupReferences(list, indices);
  normals:= BuildNormals(list, indices);
  strips:= StripifyMesh(indices, list.Count, True);
  //QueryPerformanceCounter(perfStop);
  //QueryPerformanceFrequency(perfFreq);
  MeshObjects.Clear;

   // the material library is used for coloring strips
   MaterialLibrary.Materials.Clear;

   // prepare materials (for coloring the stripes)
   for i:=0 to 30 do begin
      with MaterialLibrary.Materials.Add do begin
         Name:=IntToStr(i);
         Material.FrontProperties.Emission.Color:=VectorScale(VectorMake(i/30, (30-i)/30, (i and 27)/30), 0.3);
      end;
   end;

  obj:= TMeshObject.CreateOwned(MeshObjects);
  obj.Vertices:= list;
  obj.Mode:= momFaceGroups;
  obj.Normals:= normals;
  for i:= 0 to strips.Count - 1 do begin
    fg:= TFGVertexIndexList.CreateOwned(obj.FaceGroups);
    fg.VertexIndices:= (strips[i] as TIntegerList);
    if i > 0
      then fg.Mode:= fgmmTriangleStrip
      else fg.Mode:= fgmmTriangles;
    fg.MaterialName:= inttostr(i and 15);
  end;
  StructureChanged;
  strips.CleanFree;
  indices.Free;
  normals.Free;
  list.Free;
end;




initialization

  RegisterClass(TMyFreeForm);

//finalization

  //UnRegisterClass(TMyFreeForm);


end.
