(******************************************************************************
Copyright (C) 2003 Pascal Peter

http://pascal.peter.free.fr/
pascal.peter@free.fr

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
modifier conformment aux dispositions de la Licence Publique Gnrale GNU,
telle que publie par la Free Software Foundation ; version 2 de la licence,
ou encore toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
voir la Licence Publique Gnrale GNU.

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme
temps que ce programme ; si ce n'est pas le cas,
crivez  la Free Software Foundation Inc.,
675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
-------------------------------------------------------------------------------

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************)

unit glObjet3ds;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,

  GLScene, GLVectorFileObjects, GLMisc, GLObjects, GLBehaviours,
  GLTexture, VectorGeometry, JPeg,

  VectorLists, MeshUtils, PersistentClasses;


type

  TNomFichier = string[250];

  TObjet3ds = class(TGLDummyCube)
    private
      FNomFichier: TNomFichier;
    protected
    public
      Objet3ds_MaterialLibrary: TGLMaterialLibrary;
      AOwnerComponent: TComponent;

      FreeFormVue: TGLFreeForm;
      Encadrement: TGLDummyCube;

      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;

      procedure Initialiser;
      procedure LoadFromFile(const filename : String);
      procedure MetAJour;

      procedure Stripify;
    published
      property NomFichier: TNomFichier read FNomFichier write FNomFichier stored True;
    end;


implementation

constructor TObjet3ds.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Objet3ds_MaterialLibrary:= TGLMaterialLibrary.create(AOwner);
  AOwnerComponent:= AOwner;

  Encadrement:= TGLDummyCube(AddNewChild(TGLDummyCube));
  //Encadrement.VisibleAtRunTime:= True;
  Encadrement.Tag:= -1;

  FreeFormVue:= TGLFreeForm(AddNewChild(TGLFreeForm));
  with FreeFormVue do begin
    Visible:= True;
    PitchAngle:= 90;
    MaterialLibrary:= Objet3ds_MaterialLibrary;
    AutoCentering:=[macCenterX, macCenterY];
    end;
end;


destructor TObjet3ds.Destroy;
begin
  Initialiser;
  FreeFormVue.free;
  Objet3ds_MaterialLibrary.Free;
  DeleteChildren;
  inherited Destroy;
end;


procedure TObjet3ds.Initialiser;
begin
  FreeFormVue.MeshObjects.Clear;
  FreeFormVue.StructureChanged;
  Objet3ds_MaterialLibrary.Materials.Clear;
end;

procedure TObjet3ds.LoadFromFile(const filename : String);
begin
  Initialiser;
  //ChDir(ExtractFilePath(FileName));
  FreeFormVue.LoadFromFile(filename);
  FNomFichier:= ExtractFileName(filename);
  MetAJour;
end;

procedure TObjet3ds.MetAJour;
var
  min, max: TAffineVector;
begin
  FreeFormVue.GetExtents(min, max);
  Encadrement.Scale.X:= max[0] - min[0];
  Encadrement.Scale.Z:= max[1] - min[1];
  Encadrement.Scale.Y:= max[2] - min[2];
  Encadrement.Position.Y:= (min[2] + max[2]) / 2;
end;



procedure TObjet3ds.Stripify;
var
  i: Integer;
  list, normals: TAffineVectorList;
  indices: TIntegerList;
  strips: TPersistentObjectList;
  obj: TMeshObject;
  fg: TFGVertexIndexList;
  perfStart, perfStop, perfFreq: Int64;
begin
  with FreeFormVue do begin
    list:= MeshObjects.ExtractTriangles;
    QueryPerformanceCounter(perfStart);
    indices:= BuildVectorCountOptimizedIndices(list);
    RemapAndCleanupReferences(list, indices);
    normals:= BuildNormals(list, indices);
    strips:= StripifyMesh(indices, list.Count, True);
    QueryPerformanceCounter(perfStop);
    QueryPerformanceFrequency(perfFreq);
    MeshObjects.Clear;
    obj:= TMeshObject.CreateOwned(MeshObjects);
    obj.Vertices:= list;
    obj.Mode:= momFaceGroups;
    obj.Normals:= normals;
    for i:= 0 to strips.Count - 1 do begin
      fg:= TFGVertexIndexList.CreateOwned(obj.FaceGroups);
      fg.VertexIndices:= (strips[i] as TIntegerList);
      if i > 0
        then fg.Mode:= fgmmTriangleStrip
        else fg.Mode:= fgmmTriangles;
    end;
    StructureChanged;
    strips.CleanFree;
    indices.Free;
    normals.Free;
    list.Free;
    end;
end;





initialization

  RegisterClass(TObjet3ds);

//finalization

  //UnRegisterClass(TObjet3ds);


end.
