unit glRepere;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,

  GLScene, GLMisc, GLObjects, GLBehaviours,
  GLTexture, VectorGeometry, JPeg,

  GLVectorFileObjects,
  GLFileObj,
  VectorLists,
  MeshUtils,
  PersistentClasses;



type
  TEntree = class(TGLSphere)
    private
      FNumero: integer;
    protected
    public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
    published
      property Numero: integer read FNumero write FNumero stored True;
    end;
  var
    NumeroDeLEntree: integer;
  procedure AfficheEntrees(World: TGLDummyCube; visibilite: boolean);
  function RetourneEntree(World: TGLDummyCube; num: integer): TEntree;
  function NombreDEntrees(World: TGLDummyCube): integer;

type
  TRepere = class(TGLSphere)
    private
      FTexte: string;
    protected
      //procedure SetTexte(newTexte: string);
    public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
    published
      property Texte: string read FTexte write FTexte stored True;
    end;
  procedure AfficheReperes(World: TGLDummyCube; visibilite: boolean);

type
  TPassage = class(TGLSphere)
    private
      FNiveau: integer;
      FNumeroDEntree: integer;
    protected
    public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
    published
      property Niveau: integer read FNiveau write FNiveau stored True;
      property NumeroDEntree: integer read FNumeroDEntree write FNumeroDEntree stored True;
    end;
  procedure AffichePassages(World: TGLDummyCube; visibilite: boolean);

  procedure AfficheEntreesReperesPassages(World: TGLDummyCube; visibilite: boolean);


implementation

uses   Utils;

//********************************************
constructor TEntree.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Material.FrontProperties.Diffuse.Color:= clrRed;
end;

destructor TEntree.Destroy;
begin
  inherited Destroy;
end;

procedure AfficheEntrees(World: TGLDummyCube; visibilite: boolean);
var
  i: Integer;
  Temp: TGLBaseSceneObject;
begin
  if World.Count > 0 then
    for i:= World.Count - 1 downto 0 do begin
      Temp:= World.Children[i];
      if Temp is TEntree then
        Temp.Visible:= visibilite;
      end;
end;

function RetourneEntree(World: TGLDummyCube; num: integer): TEntree;
var
  i, j: Integer;
  Temp: TGLBaseSceneObject;
begin
  Result:= nil;
  if World.Count > 0 then
    for i:= World.Count - 1 downto 0 do begin
      Temp:= World.Children[i];
      if Temp is TEntree then begin
        j:= TEntree(Temp).Numero;
        if j = num then begin
          Result:= TEntree(Temp);
          exit;
          end;
        end;
      end;
end;

function NombreDEntrees(World: TGLDummyCube): integer;
var
  i, j: Integer;
  Temp: TGLBaseSceneObject;
begin
  j:= 0;
  if World.Count > 0 then
    for i:= World.Count - 1 downto 0 do begin
      Temp:= World.Children[i];
      if Temp is TEntree then j:= j + 1;
      end;
  Result:= j;
end;





//********************************************
constructor TRepere.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Material.FrontProperties.Diffuse.Color:= clrBlue;
end;

destructor TRepere.Destroy;
begin
  inherited Destroy;
end;

{
procedure TRepere.SetTexte(newTexte: string);
begin
  if newTexte <> FTexte then begin
    FTexte:= newTexte;
end;
}

procedure AfficheReperes(World: TGLDummyCube; visibilite: boolean);
var
  i: Integer;
  Temp: TGLBaseSceneObject;
begin
  if World.Count > 0 then
    for i:= World.Count - 1 downto 0 do begin
      Temp:= World.Children[i];
      if Temp is TRepere then
        Temp.Visible:= visibilite;
      end;
end;


//********************************************
constructor TPassage.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Material.FrontProperties.Diffuse.Color:= clrYellow;
end;

destructor TPassage.Destroy;
begin
  inherited Destroy;
end;

procedure AffichePassages(World: TGLDummyCube; visibilite: boolean);
var
  i: Integer;
  Temp: TGLBaseSceneObject;
begin
  if World.Count > 0 then
    for i:= World.Count - 1 downto 0 do begin
      Temp:= World.Children[i];
      if Temp is TPassage then
        Temp.Visible:= visibilite;
      end;
end;



//********************************************
procedure AfficheEntreesReperesPassages(World: TGLDummyCube; visibilite: boolean);
var
  i: Integer;
  Temp: TGLBaseSceneObject;
begin
  if World.Count > 0 then
    for i:= World.Count - 1 downto 0 do begin
      Temp:= World.Children[i];
      if (Temp is TEntree) or (Temp is TRepere) or (Temp is TPassage)
        then Temp.Visible:= visibilite;
      end;
end;


//********************************************
initialization

  RegisterClass(TRepere);
  RegisterClass(TEntree);
  RegisterClass(TPassage);

//finalization

  //UnRegisterClass(TObjetobj);


end.
