
unit object2xml;

interface


uses Classes, SysUtils, GmXml, dialogs;

const
  ParseBufSize = 4096;
  LineBreak = #13#10;


type

  //Should be used internally only as an override object to expose WritePrefix
  TWriterCracker = class(TWriter)
  public
    procedure DoWritePrefix(Flags: TFilerFlags; AChildPos: Integer);
  end;

  procedure ObjectBinary2XML(Input, Output: TStream);
  procedure ObjectXML2Binary(Input, Output: TStream);

  procedure ObjectBinary2GmXML(Input: TStream; var GmXML: TGmXML);
  procedure XMLNode2ObjectBinary(XML: TGmXML; GmXmlNode: TGmXmlNode; Output: TStream);
  procedure XMLNodeChildrens2ObjectBinary(XML: TGmXML; GmXmlNode: TGmXmlNode; Output: TStream);

implementation


procedure ReplaceText(var AText: string; AFind, AReplace: string);
var
  Index: integer;
begin
  while Pos(AFind, AText) <> 0 do
  begin
    Index := Pos(AFind, AText);
    Delete(AText, Index, Length(AFind));
    Insert(AReplace, AText, Index);
  end;
end;

function Html2Text(const AText: string): string;

    procedure StrReplaceChar(var AText: string);
    var
      Index, IndexEnd, i, n: Integer;
      a: string;
    begin
      while Pos('&#', AText) <> 0 do begin
        Index:= Pos('&#', AText);
        Delete(AText, Index, 2);
        IndexEnd:= Pos(';', AText);
        a:= '';
        for i:= Index to IndexEnd - 1 do a:= a + AText[i];
        Delete(AText, Index, Length(a) + 1);
        a:= Chr(StrToInt(a));
        Insert(a, AText, Index);
        end;
    end;

var
  S: string;
begin
  S:= AText;
  ReplaceText(S, '&amp;', '&');    //#38
  ReplaceText(S, '&lt;', '<');     //#60
  ReplaceText(S, '&gt;', '>');     //#62
  ReplaceText(S, '&quot;', '"');   //#34
  ReplaceText(S, '&apos;', '''');   //#39
  StrReplaceChar(S);
  Result:= S;
end;

function Text2Html(const AText: string): string;

    function ReplaceStr(const S: String; C: Char; const Replace: String): String;
    var
      I : Integer;
    begin
      Result:= '';
      for I:= Length(S) downto 1 do
        if S[i]= C then
          Result:= Replace + Result
        else
          Result:= S[i] + Result;
    end;

    type TCharSet = set of Char;
    function CharReplaceStr(const S: String; Chars: TCharSet): String;
    var
      I : Integer;
    begin
      Result:= '';
      for I:= Length(S) downto 1 do
        if not (S[i] in Chars) then
          Result:= '&#' + IntToStr(Ord(S[i])) + ';' + Result
        else
          Result:= S[i] + Result;
    end;

var
  S: string;
begin
  S:= AText;
  S:= ReplaceStr(S, '&',  '&amp;');    //#38
  S:= ReplaceStr(S, '<',  '&lt;');     //#60
  S:= ReplaceStr(S, '>',  '&gt;');     //#62
  S:= ReplaceStr(S, '"',  '&quot;');   //#34
  S:= ReplaceStr(S, '''', '&apos;');   //#39
  S:= CharReplaceStr(S, [#32..#128]);
  Result:= S;
end;







procedure ObjectBinary2XML(Input, Output: TStream);
var //SaveSeparator : Char;
    Reader : TReader;
    Writer : TWriter;
    NestingLevel : Integer;
    Encoding : String;

  procedure ConvertObject; forward;
  procedure ConvertHeader(var CloseTag : String); forward;
  procedure ConvertProperty; forward;
  procedure ConvertValue(const PropName : String); forward;


  procedure WriteIndent; //WriteIndent optimized by Wayne Sherman Dec 27, 2000
  var Spaces: string;
  begin
    Spaces:= StringOfChar(' ', NestingLevel shl 1);
    Writer.Write(Pointer(Spaces)^, Length(Spaces));
  end;

  procedure WriteStr(const S: string);
  begin
    Writer.Write(Pointer(S)^, Length(S)); //Typecast string to write, changed Dec 27 2000: Thanks to Wayne Sherman
  end;

  procedure ConvertObject;
  var CloseTag : String;
  begin  
    ConvertHeader(CloseTag);
    Inc(NestingLevel);
    while not Reader.EndOfList do ConvertProperty;
    Reader.ReadListEnd;
    while not Reader.EndOfList do ConvertObject;
    Reader.ReadListEnd;
    Dec(NestingLevel);
    WriteIndent;
    WriteStr(CloseTag);
  end;

  procedure ConvertHeader(var CloseTag : String);
  var
    Flags: TFilerFlags;
    Order: Integer;
    ClassType : String;
    ClassName : String;
    ObjectName : string;
  begin
    Reader.ReadPrefix(Flags, Order);
    ClassName := Reader.ReadStr;
    ObjectName := Reader.ReadStr;
    WriteIndent;
    WriteStr('<' + ClassName + ' ');
    CloseTag:= '</' + ClassName + '>' + LineBreak;
    if (ffInherited in Flags) then
    begin
      ClassType:= 'inherited';
    end
    else if (ffInLine in Flags) then
    begin
      ClassType:= 'inline';
    end
    else
    begin
      ClassType:= 'object';
    end;

    if ObjectName <> '' then
    begin
      WriteStr('name="'+ObjectName+'" ');
    end;
    WriteStr('type="'+ClassType+'"');
    if (ffChildPos in Flags) then
    begin
      WriteStr(' order="' + IntToStr(Order) + '"');
    end;
    WriteStr('>' + LineBreak);
  end;

  procedure ConvertProperty;
  var
    n : String;
  begin
    WriteIndent;
    n := Reader.ReadStr;
    WriteStr('<' + n);
    ConvertValue(n);
    WriteStr('</' + n + '>' + LineBreak);
  end;

  procedure ConvertValue(const PropName : String);
  const
    LineLength = 64;
    BytesPerLine = 32;
  var
    I, L, Count: Integer;
    S, T: String;
    W: WideString;
    MultiLine: Boolean;
    Buffer: array[0..BytesPerLine - 1] of Char;
    Text: array[0..BytesPerLine * 2 - 1] of Char;
  begin

    case Reader.NextValue of
      vaBinary       : t := 'binary';
      vaCollection   : t := 'collection';
      vaCurrency     : t := 'currency';
      vaDate	     : t := 'date';
      vaExtended     : t := 'float';
      vaFalse	     : t := 'boolean';
      vaIdent	     : t := 'identifier';
      vaInt8	     : t := 'shortint';
      vaInt16	     : t := 'smallint';
      vaInt32	     : t := 'longint';
      vaList	     : t := 'list';
      vaLString      : t := 'longstring';
      vaNil	     : t := 'nil';
      vaNull	     : t := 'null';
      vaSet	     : t := 'set';
      vaSingle       : t := 'single';
      vaString       : t := 'string';
      vaTrue	     : t := 'boolean';
      vaWString      : t := 'widestring';
      else             t := 'unknown';
    end;

    if (t = 'unknown') and (PropName = 'NomFichier') then t:= 'string';
    WriteStr(' type="' + t + '">');

    case Reader.NextValue of
      vaInt8, vaInt16, vaInt32:
        begin
          WriteStr(IntToStr(Reader.ReadInteger));
        end;
      vaExtended:
        begin
          WriteStr(FloatToStr(Reader.ReadFloat));
        end;
      vaSingle:
        begin
          WriteStr(FloatToStr(Reader.ReadSingle));
        end;
      vaCurrency:
        begin
          WriteStr(FloatToStr(Reader.ReadCurrency * 10000));
        end;
      vaDate:
        begin
          WriteStr(FormatDateTime('c tt', Reader.ReadDate))
        end;
      vaWString:
        begin
          W := Reader.ReadWideString;
          L := Length(W);
          if L > 0 then
          begin
            WriteStr(W);
          end;
        end;
      vaIdent, vaFalse, vaTrue, vaNil, vaNull:
        begin
          S := Reader.ReadIdent;
          WriteStr(S);
        end;
      vaSet:
        begin
          Reader.CheckValue(vaSet);
          WriteStr('[');
          I := 0;
          while True do
          begin
            S := Reader.ReadStr;
            if S = '' then Break;
            if I > 0 then WriteStr(', ');
            WriteStr(S);
            Inc(I);
          end;
          WriteStr(']');
        end;
      vaList:
        begin
          Reader.CheckValue(vaList);
          Inc(NestingLevel);
          WriteStr(LineBreak);
          while not Reader.EndOfList do
          begin
            WriteIndent;
            WriteStr('<' + PropName + '.item');
            ConvertValue(PropName + '.item');
            WriteStr('</' + PropName + '.item>' + LineBreak);
          end;
          Reader.ReadListEnd;
          Dec(NestingLevel);
          WriteIndent;
        end;
      vaCollection:
        begin
          Reader.CheckValue(vaCollection);
          Inc(NestingLevel);
          WriteStr(LineBreak);
          while not Reader.EndOfList do
          begin
            WriteStr('<' + PropName + '.item>' + LineBreak);
            Inc(NestingLevel);
            Reader.ReadListBegin;
            while not Reader.EndOfList do
            begin
              ConvertProperty;
            end;
            Reader.ReadListEnd;
            Dec(NestingLevel);
            WriteStr('</' + PropName+ '.item>' + LineBreak);
          end;
          Reader.ReadListEnd;
          Dec(NestingLevel);
          WriteIndent;
        end;
      vaBinary:
        begin
          Reader.CheckValue(vaBinary);
          Reader.Read(Count, SizeOf(Count));
          MultiLine := Count >= BytesPerLine;
          Inc(NestingLevel);
          while Count > 0 do
          begin
            if MultiLine then WriteStr(LineBreak);
            if Count >= 32 then I := 32 else I := Count;
            Reader.Read(Buffer, I);
            BinToHex(Buffer, Text, I);
            Writer.Write(Text, I * 2);
            Dec(Count, I);
          end;
          Dec(NestingLevel);
          if MultiLine then WriteStr(LineBreak);

        end;
      vaString, vaLString:
        begin
          S := Reader.ReadString;
          L := Length(S);
          if L > 0 then begin
            if (S[L] = #0) then Delete(S, L, 1);
            S:= Text2Html(S);
            end;

          WriteStr(S);
        end;
      else begin
        S:= Reader.ReadString;
        S:= Text2Html(S);
        WriteStr(S);
        end;
    end;
  end;

begin
  NestingLevel := 0;
  Reader := TReader.Create(Input, ParseBufSize);
  //SaveSeparator := DecimalSeparator;
  //DecimalSeparator := '.';

  try
    Writer := TWriter.Create(Output, ParseBufSize);
    try
      Reader.ReadSignature;
      {$IFDEF DEFAULTENCODING}
      Encoding:= '<?xml version="1.0"?>';
      {$ENDIF}
      {$IFDEF UTF8}
      Encoding:= '<?xml version="1.0" encoding="UTF-8"?>';
      {$ENDIF}
      {$IFDEF UTF16}
      Encoding:= '<?xml version="1.0" encoding="UTF-16"?>';
      {$ENDIF}
      {$IFDEF ISO88591}
      Encoding:= '<?xml version="1.0" encoding="ISO-8859-1"?>';
      {$ENDIF}
      WriteStr(Encoding + LineBreak);
      ConvertObject;
    finally
      Writer.Free;
    end;
  finally
    //DecimalSeparator := SaveSeparator;
    Reader.Free;
  end;
end;

procedure ObjectBinary2GmXML(Input: TStream; var GmXML: TGmXML);
var
  XML: TStringStream;
  ss1: string;
begin
  XML:= TStringStream.Create(ss1);
  try
    Input.Position:= 0;
    ObjectBinary2XML(Input, XML);
    GmXML.Nodes.AddXMLString(XML.datastring);
  finally
    XML.Free;
    end;
end;

procedure XMLNode2ObjectBinary(XML: TGmXML; GmXmlNode: TGmXmlNode; Output: TStream);
var
  a: string;
  ssXML: TStringStream;
begin
  a:= XML_SPECIFICATION + #13#10;
  XML.NodeToString(a, GmXmlNode);
  ssXML:= TStringStream.Create(a);
  try
    ObjectXML2Binary(ssXML, Output);
  finally
    ssXML.Free;
    end;
end;

procedure XMLNodeChildrens2ObjectBinary(XML: TGmXML; GmXmlNode: TGmXmlNode; Output: TStream);
var
  i, j: integer;
  a: string;
  ssXML: TStringStream;
begin
  i:= GmXmlNode.Children.count;
  a:= XML_SPECIFICATION + #13#10;
  if i > 0 then
    for j:= 0 to i - 1 do
      XML.NodeToString(a, GmXmlNode.Children.Node[j]);
  ssXML:= TStringStream.Create(a);
  try
    ObjectXML2Binary(ssXML, Output);
  finally
    ssXML.Free;
    end;
end;


























































































procedure ObjectXML2Binary(Input, Output: TStream);
var Writer : TWriterCracker;
    Strings : TStrings;
    XmlDocument : TGmXML;
    //SaveSeparator: Char;

  a, b: string;
  i: integer;

  procedure ConvertProperty(Node : TGmXmlNode); forward;
  procedure ConvertValue(Node : TGmXmlNode); forward;

  procedure ConvertObject(Node : TGmXmlNode);
  var XmlChildren : TGmXmlNodeList;
      XmlAttribute : TGmXmlNodeAttribute;
      IsInline : Boolean;
      IsInherited : Boolean;
      ObjectName : String;
      ClassType : String;
      Order : Integer;
      I : Integer;
      Flags: TFilerFlags;
  begin

    //Initialize
    IsInline:= False;
    IsInherited:= False;
    ObjectName:= '';
    ClassType:= '';
    Order:= -1;
    //Get the name attribute
    XmlAttribute:= Node.Attribute;
    ObjectName:= Node.GetNamedItemAttribute('name');
    ClassType:= Node.GetNamedItemAttribute('type');
    if Node.GetNamedItemAttribute('order') <> '' then
      Order:= StrToInt(Node.GetNamedItemAttribute('order'));
    //Is it inline or inherited?
    if (ClassType = 'inline') then IsInline:= True;
    if (ClassType = 'inherited') then IsInherited:= True;
    //Write the Object properties to the stream
    Flags := [];
    if IsInherited then
      Include(Flags, ffInherited);
    if IsInline then
      Include(Flags, ffInline);
    if Order >= 0 then
      Include(Flags, ffChildPos);
    Writer.DoWritePrefix(Flags, Order);
    Writer.WriteStr(Node.Name);
    Writer.WriteStr(ObjectName);
    //Acquire the Children
    XmlChildren:= Node.Children;
    //Loop through the Children
    for I:= 0 to XmlChildren.count-1 do
    begin
      //Check if it is an Element Node (NODE_ELEMENT = 1)
      //if XmlChildren[I].IsLeafNode then
      //begin
        //Get the Attribute
        ClassType:= XmlChildren[I].GetNamedItemAttribute('type');
        //Convert the Element
        if (ClassType <> '') then
        begin
          if (ClassType = 'object') or
             (ClassType = 'inline') or
             (ClassType = 'inherited') then
          begin
            //Recurse
            Writer.WriteListEnd;
            ConvertObject(XmlChildren[I]);
            Writer.WriteListEnd;
          end else
          begin
            //Read the properties
            ConvertProperty(XmlChildren[I]);
          end;
        end;
      //end;
    end;
    
  end;

  procedure ConvertProperty(Node : TGmXmlNode);
  begin
    Writer.WriteStr(Node.Name);
    ConvertValue(Node);
  end;

  procedure ConvertValue(Node : TGmXmlNode);
  const
      BytesPerLine = 32;
      LineLength = 64;
  var
      {XmlAttributes : TObjectXmlNamedNodeMap;
      XmlChildren : TObjectXmlNodeList;
      XmlChildrenCollection : TObjectXmlNodeList;   }
      XmlChildren, XmlChildrenCollection: TGmXmlNodeList;
      PropTypeName : String;
      PropType : TValueType;
      PropValue, a : String;
      StartPos, I, J, L : Integer;
      Count, TempCount : Longint;
      Buffer : array[0..BytesPerLine - 1] of Char;
      Text : array[0..LineLength - 1] of Char;
      Stream : TMemoryStream;
      S: string;
  begin
    PropType:= vaNull;

    PropTypeName:= LowerCase(Node.GetNamedItemAttribute('type'));
    if      (PropTypeName = 'binary')           then PropType:= vaBinary
    else if (PropTypeName = 'collection')       then PropType:= vaCollection
    else if (PropTypeName = 'currency')         then PropType:= vaCurrency
    else if (PropTypeName = 'date')             then PropType:= vaDate
    else if (PropTypeName = 'float')            then PropType:= vaExtended
    else if (PropTypeName = 'boolean')          then PropType:= vaFalse
    else if (PropTypeName = 'identifier')       then PropType:= vaIdent
    else if (PropTypeName = 'shortint')         then PropType:= vaInt8
    else if (PropTypeName = 'smallint')         then PropType:= vaInt16
    else if (PropTypeName = 'longint')          then PropType:= vaInt32
    else if (PropTypeName = 'list')             then PropType:= vaList
    else if (PropTypeName = 'longstring')       then PropType:= vaLString
    else if (PropTypeName = 'nil')              then PropType:= vaNil
    else if (PropTypeName = 'null')             then PropType:= vaNull
    else if (PropTypeName = 'set')              then PropType:= vaSet
    else if (PropTypeName = 'single')           then PropType:= vaSingle
    else if (PropTypeName = 'string')           then PropType:= vaString
    else if (PropTypeName = 'boolean')          then PropType:= vaTrue
    else if (PropTypeName = 'widestring')       then PropType:= vaWString
    else PropType:= vaString;

    case PropType of
      vaInt8, vaInt16, vaInt32:
        begin
          Writer.WriteInteger(StrToInt(Node.AsString));
        end;
      vaExtended:
        begin
          Writer.WriteFloat(StrToFloat(Node.AsString));
        end;
      vaSingle:
        begin
          Writer.WriteSingle(StrToFloat(Node.AsString));
        end;
      vaCurrency:
        begin
          Writer.WriteCurrency(StrToFloat(Node.AsString));
        end;
      vaDate:
        begin
          Writer.WriteDate(StrToDateTime(Node.AsString));
        end;
      vaWString:
        begin
          if (Node.AsString <> '') then
            Writer.WriteWideString(Node.AsString)
          else begin
            L:= 0;
            Writer.WriteValue(vaWString);
            Writer.Write(L, SizeOf(Byte));
          end;
        end;
      vaString, vaLString:
        begin
          S:= Node.AsString;
          if (S <> '')
            then begin
              S:= Html2Text(S);
              Writer.WriteString(S);
              end
            else begin
              L:= 0;
              Writer.WriteValue(vaString);
              Writer.Write(L, SizeOf(Byte));
              end;
        end;
      vaIdent, vaFalse, vaTrue, vaNil, vaNull:
        begin
          Writer.WriteIdent(Node.AsString);
        end;
      vaSet:
        begin
          Writer.WriteValue(vaSet);
          if (Node.AsString <> '') then
          begin
            PropValue:= Node.AsString;
            Delete(PropValue, 1, 1);
            Delete(PropValue, Length(PropValue), 1);
            if (Length(Trim(PropValue)) > 1) then
            begin
              StartPos:= 1;
              for I:= 1 to Length(PropValue) do
              begin
                if (PropValue[I] = ',') then
                begin
                  Writer.WriteStr(Trim(Copy(PropValue, StartPos, I-StartPos)));
                  StartPos:= I+1;
                end;
              end;
              Writer.WriteStr(Trim(Copy(PropValue, StartPos, Length(PropValue))));
            end;
          end;
          Writer.WriteListEnd;
        end;
      vaList:
        begin
          Writer.WriteListBegin;
          XmlChildren:= Node.Children;
          for I:= 0 to XmlChildren.count-1 do
          begin
            //Check if it is an Element Node (NODE_ELEMENT = 1)
            //if (Byte(XmlChildren[I].nodeType) = 1) then
              ConvertValue(XmlChildren[I]);
          end;
          Writer.WriteListEnd;
        end;
      vaCollection:
        begin
          Writer.WriteValue(vaCollection);
          XmlChildren:= Node.Children;
          for I:= 0 to XmlChildren.count-1 do
          begin
            XmlChildrenCollection:= XmlChildren[I].Children;
            Writer.WriteListBegin;
            for J:= 0 to XmlChildrenCollection.count-1 do
            begin
              //Check if it is an Element Node (NODE_ELEMENT = 1)
              //if (Byte(XmlChildrenCollection[J].nodeType) = 1) then
                ConvertProperty(XmlChildrenCollection[J]);
            end;
            Writer.WriteListEnd;
          end;
          Writer.WriteListEnd;
        end;
      vaBinary:
        begin
          //Get the Value
          PropValue:= Trim(Node.AsString);
          while Pos(#9, PropValue) > 0 do Delete(PropValue, Pos(#9, PropValue), 1);
          while Pos(#10, PropValue) > 0 do Delete(PropValue, Pos(#10, PropValue), 1);
          while Pos(#13, PropValue) > 0 do Delete(PropValue, Pos(#13, PropValue), 1);
          while Pos(#32, PropValue) > 0 do Delete(PropValue, Pos(#32, PropValue), 1);
          //Create a temporary Stream
          Stream:= TMemoryStream.Create;
          try
            //Erase the next line
            FillChar(Text, SizeOf(Text), 0);
            FillChar(Buffer, Sizeof(Buffer), 0);
            //Initialize the Count
            Count:= length(PropValue) div 2;
            TempCount:= 0;
            Writer.WriteValue(vaBinary);
            Writer.Write(Count, SizeOf(Count));

            //Loop through
            while (length(PropValue) > LineLength) do begin
              a:= Copy(PropValue, 1, LineLength);
              Delete(PropValue, 1, LineLength);
              FillChar(Text, SizeOf(Text), 0);
              FillChar(Buffer, Sizeof(Buffer), 0);
              for i:= 0 to LineLength - 1 do Text[i]:= a[i+1];
              TempCount:= HexToBin(Text, Buffer, SizeOf(Buffer));
              //Stream.Seek(0, soFromEnd);
              Stream.Write(Buffer, TempCount);
              Writer.Write(Buffer, TempCount);
              ////Count:= Count + TempCount;
              end;

            FillChar(Text, SizeOf(Text), 0);
            FillChar(Buffer, Sizeof(Buffer), 0);
            TempCount:= 0;
            for I:= 1 to Length(PropValue) do
            begin
              {
              if (PropValue[I] in [#9, #13, #32]) then Continue;
              if (PropValue[I] = #10) then
              begin
                //Converty the Hex to Binary
                TempCount:= HexToBin(Text, Buffer, Sizeof(Buffer)); //Changed Dec 14: Thanks to Scott Kellish
                //Write the Line
                //Stream.Seek(0, soFromEnd);
                Stream.Write(Buffer, TempCount);
                Writer.Write(Buffer, TempCount);
                ////Count:= Count + TempCount;
                //Erase the next line
                FillChar(Text, SizeOf(Text), 0);
                FillChar(Buffer, Sizeof(Buffer), 0);
                //Restart the Count
                TempCount:= 0;
                //Continue (do not write the #10)
                Continue;
              end;
              }
              Text[TempCount]:= PropValue[I];
              Inc(TempCount);
            end;

            //Write the Final Row if Count > 0
            if TempCount > 0 then
            begin
              TempCount:= HexToBin(Text, Buffer, SizeOf(Buffer)); //Changed Dec 14: Thanks to Scott Kellish
              //Write the Line
              //Stream.Seek(0, soFromEnd);
              Stream.Write(Buffer, TempCount);
              Writer.Write(Buffer, TempCount);
              ////Count:= Count + TempCount;
            end;

            Stream.Seek(0, soFromBeginning);
            TempCount:= Stream.Size;
          finally
            Stream.Free;
          end;
        end;
    end;
  end;


begin
  //SaveSeparator := DecimalSeparator;
  //DecimalSeparator := '.';
  Strings:= TStringList.Create;
  Writer := TWriterCracker.Create(Output, ParseBufSize);
  try
    Strings.LoadFromStream(Input);

    a:= '';
    if Strings.Count > 0 then
     for i:= 0 to Strings.Count - 1 do begin
       b:= Strings[i];
       ReplaceText(b, #13 + #10, '');
       a:= a + b;
       end;

    XmlDocument:= TGmXml.create(nil);
    XmlDocument.Text:= a;//Strings.Text;
    try
      Writer.WriteSignature;
      ConvertObject(XmlDocument.Nodes.Root);
      Writer.WriteListEnd;
      Writer.WriteListEnd;
    finally
    end;
  finally
    Writer.Free;
    Strings.Free;
    //DecimalSeparator := SaveSeparator;
  end;
end;

{ TWriterCracker }

procedure TWriterCracker.DoWritePrefix(Flags: TFilerFlags;
  AChildPos: Integer);
begin
  Self.WritePrefix(Flags, AChildPos);
end;

end.

